// =============================================================================
// soubor building_a07.cpp
// =============================================================================

#include"..\buildings.h"

// VERTICES 
// body v prostoru jejich propojenim vytvarim model budovy 
static float a07_vertices[][3] = {
	-2.f, 0, -0.5f,  // 0
	 2.f, 0, -0.5f,  // 1
   2.f, 2, -0.5f,  // 2
  -2.f, 2, -0.5f,  // 3

	-2.f, 2.5, 0.5f,  // 4
	 2.f, 2.5, 0.5f,  // 5

	 2.f, 2, 0.5f,  // 6
  -2.f, 2, 0.5f,  // 7
  -2.f, 0, 0.5f,  // 8
   2.f, 0, 0.5f,  // 9
};

// NORMALS
// normaly pro dobre osvetleni sten modelu budovy
static float a07_normal[][3] = {
	 1,  0,  0, // 0 x 
   0,  1,  0, // 1 y
   0,  0,  1, // 2 z
 
	-1,  0,  0, // 3 -x
   0, -1,  0, // 4 -y
   0,  0, -1, // 5 -z

   0,0.89442f,-0.44721f  // 6 front_up sklon strechy
};

// steny z bodu
static int32_t a07_indices_front[] = { 1,0,2,3, -1 };
static int32_t a07_indices_up[] = { 2,3,5,4, -1 };
static int32_t a07_indices_left[] = { 5,2,6,1,9, -1 };
static int32_t a07_indices_right[] = { 4,3,7,0,8, -1 };
static int32_t a07_indices_back[] = { 4,5,8,9, -1 };
static int32_t a07_indices_bottom[] = { 0,1,8,9, -1 };

// normaly ke stenam
static int normalIndex_a07_front[] = { 5, 5 };
static int normalIndex_a07_up[] = { 6, 6 };
static int normalIndex_a07_left[] = { 0, 0, 0 };
static int normalIndex_a07_right[] = { 3, 3, 3 };
static int normalIndex_a07_back[] = { 2, 2 };
static int normalIndex_a07_bottom[] = { 4, 4 };

// texturovaci souradnice pro jednotlive steny
static int32_t TexCoordIndex_a07_front[] = { 0,1,2,3, -1 };
static int32_t TexCoordIndex_a07_up[] = { 0,1,2,3, -1 };
static int32_t TexCoordIndex_a07_left[] = { 3,4,5,0,1, -1 };
static int32_t TexCoordIndex_a07_right[] = { 3,4,5,0,1, -1 };
static int32_t TexCoordIndex_a07_back[] = { 2,3,0,1, -1 };
static int32_t TexCoordIndex_a07_bottom[] = { 0,1,2,3, -1 };


// -----------------------------------------------------------------------------
// fce.MakeBuilding_a07 - vytvori budovu a07
// -----------------------------------------------------------------------------
SoSeparator *MakeBuilding_a07(float x, float y, float z, float alfa, TEXTURE texture) {

	// uzel oznacujici budovu
	SoSeparator *a07 = new SoSeparator();
	a07 -> ref();

	// umisteni budovy
  SoTransform *transform = new SoTransform;
	transform->translation.setValue(x,y,z);
  a07->addChild(transform);   

	// velikost modelu budovy
  SoScale *scale = new SoScale;
  scale->scaleFactor.setValue(SCALE_X,SCALE_Y,SCALE_Z);
  a07->addChild(scale);

  // rotace modelu budovy
  SoRotationXYZ *myRotation = new SoRotationXYZ;     
  myRotation -> angle = alfa;
  myRotation -> axis = SoRotationXYZ::Y;
  a07 -> addChild(myRotation);

	// normaly modelu budovy
  SoNormal *myNormals = new SoNormal;
  myNormals -> vector.setValues(0, 11, a07_normal);
  a07 -> addChild(myNormals); 

	// prirazovani normal PER_FACE_INDEXED
  SoNormalBinding *myNormalBinding = new SoNormalBinding;
  myNormalBinding -> value = SoNormalBinding::PER_FACE_INDEXED;
  a07 -> addChild(myNormalBinding);
 
	// propojeni bodu modelu budovy
  SoCoordinate3 *coords = new SoCoordinate3;
  coords->point.setValues(0, sizeof(a07_vertices)/sizeof(float), a07_vertices);
  a07->addChild(coords);

	// 'vyrez textury' (tvar 2D pro krychly potrebujeme ctverec bereme celou textuturu)' 
  SoTextureCoordinate2 *texCoord = new SoTextureCoordinate2;
  texCoord->point.set1Value(0, SbVec2f(0,0));
  texCoord->point.set1Value(1, SbVec2f(1,0));
  texCoord->point.set1Value(2, SbVec2f(0,1));
  texCoord->point.set1Value(3, SbVec2f(1,1));
  texCoord->point.set1Value(4, SbVec2f(0,0.6666f));
  texCoord->point.set1Value(5, SbVec2f(1,0.6666f));

  a07->addChild(texCoord);

  // front
  SoTexture2 *texture_a07 = new SoTexture2;
  a07 -> addChild(texture_a07);
	switch(texture) {
		case BRICK: texture_a07 -> filename.setValue("Texture/buildings/a07/brick/texture_a07_front.PNG"); break;
		case COLOR: texture_a07 -> filename.setValue("Texture/buildings/a07/color/texture_a07_front.PNG"); break;
		case STONE: texture_a07 -> filename.setValue("Texture/buildings/a07/stone/texture_a07_front.PNG"); break;
		case WOOD:  texture_a07 -> filename.setValue("Texture/buildings/a07/wood/texture_a07_front.PNG");  break;
	}


  SoIndexedTriangleStripSet *stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(a07_indices_front)/sizeof(int32_t), a07_indices_front);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_a07_front)/sizeof(int), normalIndex_a07_front);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_a07_front)/sizeof(int32_t), TexCoordIndex_a07_front); 
  a07->addChild(stripSet);


  // up
  texture_a07 = new SoTexture2;
  a07 -> addChild(texture_a07);
	switch(texture) {
		case BRICK: texture_a07 -> filename.setValue("Texture/buildings/a07/brick/texture_a07_up.PNG"); break;
		case COLOR: texture_a07 -> filename.setValue("Texture/buildings/a07/color/texture_a07_up.PNG"); break;
		case STONE: texture_a07 -> filename.setValue("Texture/buildings/a07/stone/texture_a07_up.PNG"); break;
		case WOOD:  texture_a07 -> filename.setValue("Texture/buildings/a07/wood/texture_a07_up.PNG");  break;
	}


  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(a07_indices_up)/sizeof(int32_t), a07_indices_up);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_a07_up)/sizeof(int), normalIndex_a07_up);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_a07_up)/sizeof(int32_t), TexCoordIndex_a07_up); 
  a07->addChild(stripSet);


  // left
  texture_a07 = new SoTexture2;
  a07 -> addChild(texture_a07);
	switch(texture) {
		case BRICK: texture_a07 -> filename.setValue("Texture/buildings/a07/brick/texture_a07_left.PNG"); break;
		case COLOR: texture_a07 -> filename.setValue("Texture/buildings/a07/color/texture_a07_left.PNG"); break;
		case STONE: texture_a07 -> filename.setValue("Texture/buildings/a07/stone/texture_a07_left.PNG"); break;
		case WOOD:  texture_a07 -> filename.setValue("Texture/buildings/a07/wood/texture_a07_left.PNG");  break;
	}


  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(a07_indices_left)/sizeof(int32_t), a07_indices_left);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_a07_left)/sizeof(int), normalIndex_a07_left);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_a07_left)/sizeof(int32_t), TexCoordIndex_a07_left); 
  a07->addChild(stripSet);


  // right
  texture_a07 = new SoTexture2;
  a07 -> addChild(texture_a07);
	switch(texture) {
		case BRICK: texture_a07 -> filename.setValue("Texture/buildings/a07/brick/texture_a07_right.PNG"); break;
		case COLOR: texture_a07 -> filename.setValue("Texture/buildings/a07/color/texture_a07_right.PNG"); break;
		case STONE: texture_a07 -> filename.setValue("Texture/buildings/a07/stone/texture_a07_right.PNG"); break;
		case WOOD:  texture_a07 -> filename.setValue("Texture/buildings/a07/wood/texture_a07_right.PNG");  break;
	}


  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(a07_indices_right)/sizeof(int32_t), a07_indices_right);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_a07_right)/sizeof(int), normalIndex_a07_right);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_a07_right)/sizeof(int32_t), TexCoordIndex_a07_right); 
  a07->addChild(stripSet);

  
  // back
  texture_a07 = new SoTexture2;
  a07 -> addChild(texture_a07);
	switch(texture) {
		case BRICK: texture_a07 -> filename.setValue("Texture/buildings/a07/brick/texture_a07_back.PNG"); break;
		case COLOR: texture_a07 -> filename.setValue("Texture/buildings/a07/color/texture_a07_back.PNG"); break;
		case STONE: texture_a07 -> filename.setValue("Texture/buildings/a07/stone/texture_a07_back.PNG"); break;
		case WOOD:  texture_a07 -> filename.setValue("Texture/buildings/a07/wood/texture_a07_back.PNG");  break;
	}


  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(a07_indices_back)/sizeof(int32_t), a07_indices_back);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_a07_back)/sizeof(int), normalIndex_a07_back);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_a07_back)/sizeof(int32_t), TexCoordIndex_a07_back); 
  a07->addChild(stripSet);


  // bottom
  texture_a07 = new SoTexture2;
  a07 -> addChild(texture_a07);
  texture_a07 -> filename.setValue("texture_bottom.PNG");

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(a07_indices_bottom)/sizeof(int32_t), a07_indices_bottom);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_a07_bottom)/sizeof(int), normalIndex_a07_bottom);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_a07_bottom)/sizeof(int32_t), TexCoordIndex_a07_bottom); 
  a07->addChild(stripSet);
 

	a07 -> unrefNoDelete();
	return a07;
}
// -----------------------------------------------------------------------------


// =============================================================================
// konec souboru building_a07.cpp
// =============================================================================