// =============================================================================
// soubor building_f01.cpp
// =============================================================================

#include"..\buildings.h"

// VERTICES 
// body v prostoru jejich propojenim vytvarim model budovy 
static float f01_vertices[][3] = {
	-1, 0, -0.5f,  // 0
	 1, 0, -0.5f,  // 1
  -1, 1, -0.5f,  // 2
   1, 1, -0.5f,  // 3

	-1, 0,  0.5f,  // 4
   1, 0,  0.5f,  // 5
  -1, 1,  0.5f,  // 6
   1, 1,  0.5f   // 7
};

// NORMALS
// normaly pro dobre osvetleni sten modelu budovy
static float f01_normal[][3] = {
	 1,  0,  0, // 0 x 
   0,  1,  0, // 1 y
   0,  0,  1, // 2 z
 
	-1,  0,  0, // 3 -x
   0, -1,  0, // 4 -y
   0,  0, -1, // 5 -z
};

// steny z bodu
static int32_t f01_indices_front[] = {0,1,2,3, -1};
static int32_t f01_indices_left[] = {0,2,4,6, -1};
static int32_t f01_indices_right[] = {1,3,5,7, -1};
static int32_t f01_indices_back[] = {4,5,6,7, -1};
static int32_t f01_indices_up[] = {2,3,6,7, -1};
static int32_t f01_indices_bottom[] = {0,1,4,5, -1};

// normaly ke stenam
static int normalIndex_f01_front[] = {5,5};   // -z     
static int normalIndex_f01_left[] = {3,3};    // -x
static int normalIndex_f01_right[] = {0,0};   //  x
static int normalIndex_f01_back[] = {2,2};    //  z
static int normalIndex_f01_up[] = {1,1};      //  y  
static int normalIndex_f01_bottom[] = {4,4};  // -y

// texturovaci souradnice pro jednotlive steny
static int32_t TexCoordIndex_f01_front[] = {0,1,2,3, -1};
static int32_t TexCoordIndex_f01_up[] = {0,1,2,3, -1};
static int32_t TexCoordIndex_f01_left[] = {0,2,1,3, -1};
static int32_t TexCoordIndex_f01_right[] = {0,2,1,3, -1};
static int32_t TexCoordIndex_f01_back[] = {0,1,2,3, -1};
static int32_t TexCoordIndex_f01_bottom[] = {0,1,2,3, -1};


// -----------------------------------------------------------------------------
// fce.MakeBuilding_f01 - vytvori budovu f01
// -----------------------------------------------------------------------------
SoSeparator *MakeBuilding_f01(float x, float y, float z, float alfa, TEXTURE texture) {

  // uzel oznacujici budovu
  SoSeparator *f01 = new SoSeparator;
  f01 -> ref();

  // umisteni budovy
  SoTransform *transform = new SoTransform;
  transform->translation.setValue(x,y,z);
  f01->addChild(transform);

  // velikost modelu budovy 
  SoScale *scale = new SoScale;
  scale->scaleFactor.setValue(SCALE_X,SCALE_Y,SCALE_Z);
  f01->addChild(scale);

  // rotace modelu budovy 
  SoRotationXYZ *myRotation = new SoRotationXYZ;     
  myRotation -> angle = alfa;
  myRotation -> axis = SoRotationXYZ::Y;
  f01 -> addChild(myRotation);

  // normaly modelu budovy
  SoNormal *myNormals = new SoNormal;
  myNormals -> vector.setValues(0, 10, f01_normal);
  f01 -> addChild(myNormals); 

  // prirazovani normal PER_FACE_INDEXED 
  SoNormalBinding *myNormalBinding = new SoNormalBinding;
  myNormalBinding -> value = SoNormalBinding::PER_FACE_INDEXED;
  f01 -> addChild(myNormalBinding);

  // propojeni bodu modelu budovy 
  SoCoordinate3 *coords = new SoCoordinate3;
  coords->point.setValues(0, sizeof(f01_vertices)/sizeof(float), f01_vertices);
  f01->addChild(coords);

  // 'vyrez textury' (tvar 2D pro krychly potrebujeme ctverec bereme celou textuturu)' 
  SoTextureCoordinate2 *texCoord = new SoTextureCoordinate2;
  texCoord->point.set1Value(0, SbVec2f(0,0));
  texCoord->point.set1Value(1, SbVec2f(1,0));
  texCoord->point.set1Value(2, SbVec2f(0,1));
  texCoord->point.set1Value(3, SbVec2f(1,1));
  f01->addChild(texCoord);

  // front
  SoTexture2 *texture_f01 = new SoTexture2;
  f01 -> addChild(texture_f01);
	switch(texture) {
		case BRICK: texture_f01 -> filename.setValue("Texture/buildings/f01/brick/texture_f01_front.PNG"); break;
		case COLOR: texture_f01 -> filename.setValue("Texture/buildings/f01/color/texture_f01_front.PNG"); break;
		case STONE: texture_f01 -> filename.setValue("Texture/buildings/f01/stone/texture_f01_front.PNG"); break;
		case WOOD:  texture_f01 -> filename.setValue("Texture/buildings/f01/wood/texture_f01_front.PNG");  break;
	}

  SoIndexedTriangleStripSet *stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(f01_indices_front)/sizeof(int32_t), f01_indices_front);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_f01_front)/sizeof(int), normalIndex_f01_front);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_f01_front)/sizeof(int32_t), TexCoordIndex_f01_front); 
  f01->addChild(stripSet);


  // up
  texture_f01 = new SoTexture2;
  f01 -> addChild(texture_f01);
	switch(texture) {
		case BRICK: texture_f01 -> filename.setValue("Texture/buildings/f01/brick/texture_f01_up.PNG"); break;
		case COLOR: texture_f01 -> filename.setValue("Texture/buildings/f01/color/texture_f01_up.PNG"); break;
		case STONE: texture_f01 -> filename.setValue("Texture/buildings/f01/stone/texture_f01_up.PNG"); break;
		case WOOD:  texture_f01 -> filename.setValue("Texture/buildings/f01/wood/texture_f01_up.PNG");  break;
	}


  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(f01_indices_up)/sizeof(int32_t), f01_indices_up);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_f01_up)/sizeof(int), normalIndex_f01_up);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_f01_up)/sizeof(int32_t), TexCoordIndex_f01_up); 
  f01->addChild(stripSet);


  // left
  texture_f01 = new SoTexture2;
  f01 -> addChild(texture_f01);
	switch(texture) {
		case BRICK: texture_f01 -> filename.setValue("Texture/buildings/f01/brick/texture_f01_left.PNG"); break;
		case COLOR: texture_f01 -> filename.setValue("Texture/buildings/f01/color/texture_f01_left.PNG"); break;
		case STONE: texture_f01 -> filename.setValue("Texture/buildings/f01/stone/texture_f01_left.PNG"); break;
		case WOOD:  texture_f01 -> filename.setValue("Texture/buildings/f01/wood/texture_f01_left.PNG");  break;
	}

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(f01_indices_left)/sizeof(int32_t), f01_indices_left);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_f01_left)/sizeof(int), normalIndex_f01_left);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_f01_left)/sizeof(int32_t), TexCoordIndex_f01_left); 
  f01->addChild(stripSet);


  // right
  texture_f01 = new SoTexture2;
  f01 -> addChild(texture_f01);
	switch(texture) {
		case BRICK: texture_f01 -> filename.setValue("Texture/buildings/f01/brick/texture_f01_right.PNG"); break;
		case COLOR: texture_f01 -> filename.setValue("Texture/buildings/f01/color/texture_f01_right.PNG"); break;
		case STONE: texture_f01 -> filename.setValue("Texture/buildings/f01/stone/texture_f01_right.PNG"); break;
		case WOOD:  texture_f01 -> filename.setValue("Texture/buildings/f01/wood/texture_f01_right.PNG");  break;
	}

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(f01_indices_right)/sizeof(int32_t), f01_indices_right);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_f01_right)/sizeof(int), normalIndex_f01_right);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_f01_right)/sizeof(int32_t), TexCoordIndex_f01_right); 
  f01->addChild(stripSet);

  
  // back
  texture_f01 = new SoTexture2;
  f01 -> addChild(texture_f01);
	switch(texture) {
		case BRICK: texture_f01 -> filename.setValue("Texture/buildings/f01/brick/texture_f01_back.PNG"); break;
		case COLOR: texture_f01 -> filename.setValue("Texture/buildings/f01/color/texture_f01_back.PNG"); break;
		case STONE: texture_f01 -> filename.setValue("Texture/buildings/f01/stone/texture_f01_back.PNG"); break;
		case WOOD:  texture_f01 -> filename.setValue("Texture/buildings/f01/wood/texture_f01_back.PNG");  break;
	}

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(f01_indices_back)/sizeof(int32_t), f01_indices_back);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_f01_back)/sizeof(int), normalIndex_f01_back);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_f01_back)/sizeof(int32_t), TexCoordIndex_f01_back); 
  f01->addChild(stripSet);


  // bottom
  texture_f01 = new SoTexture2;
  f01 -> addChild(texture_f01);
  texture_f01 -> filename.setValue("texture_bottom.PNG");

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(f01_indices_bottom)/sizeof(int32_t), f01_indices_bottom);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_f01_bottom)/sizeof(int), normalIndex_f01_bottom);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_f01_bottom)/sizeof(int32_t), TexCoordIndex_f01_bottom); 
  f01->addChild(stripSet);

  f01 -> unrefNoDelete();

  return f01;
} 
// ----------------------------------------------------------------------------- 


// =============================================================================
// konec souboru building_f01.cpp
// =============================================================================