// =============================================================================
// soubor building_f05.cpp
// =============================================================================

#include"..\buildings.h"

// VERTICES 
// body v prostoru jejich propojenim vytvarim model budovy 
static float f05_vertices[][3] = {
	-0.5f, 0, -0.5f,  // 0
	 0.5f, 0, -0.5f,  // 1
  -0.5f, 3, -0.5f,  // 2
   0.5f, 3, -0.5f,  // 3

	-0.5f, 0,  0.5f,  // 4
   0.5f, 0,  0.5f,  // 5
  -0.5f, 3,  0.5f,  // 6
   0.5f, 3,  0.5f   // 7
};

// NORMALS
// normaly pro dobre osvetleni sten modelu budovy
static float f05_normal[][3] = {
	 1,  0,  0, // 0 x 
   0,  1,  0, // 1 y
   0,  0,  1, // 2 z
 
	-1,  0,  0, // 3 -x
   0, -1,  0, // 4 -y
   0,  0, -1, // 5 -z
};

// steny z bodu
static int32_t f05_indices_front[] = {0,1,2,3, -1};
static int32_t f05_indices_left[] = {0,2,4,6, -1};
static int32_t f05_indices_right[] = {1,3,5,7, -1};
static int32_t f05_indices_back[] = {4,5,6,7, -1};
static int32_t f05_indices_up[] = {2,3,6,7, -1};
static int32_t f05_indices_bottom[] = {0,1,4,5, -1};

// normaly ke stenam
static int normalIndex_f05_front[] = {5,5};   // -z     
static int normalIndex_f05_left[] = {3,3};    // -x
static int normalIndex_f05_right[] = {0,0};   //  x
static int normalIndex_f05_back[] = {2,2};    //  z
static int normalIndex_f05_up[] = {1,1};      //  y  
static int normalIndex_f05_bottom[] = {4,4};  // -y

// texturovaci souradnice pro jednotlive steny
static int32_t TexCoordIndex_f05_front[] = {0,1,2,3, -1};
static int32_t TexCoordIndex_f05_up[] = {0,1,2,3, -1};
static int32_t TexCoordIndex_f05_left[] = {0,2,1,3, -1};
static int32_t TexCoordIndex_f05_right[] = {0,2,1,3, -1};
static int32_t TexCoordIndex_f05_back[] = {0,1,2,3, -1};
static int32_t TexCoordIndex_f05_bottom[] = {0,1,2,3, -1};


// -----------------------------------------------------------------------------
// fce.MakeBuilding_f05 - vytvori budovu f05
// -----------------------------------------------------------------------------
SoSeparator *MakeBuilding_f05(float x, float y, float z, float alfa, TEXTURE texture) {

  // uzel oznacujici budovu
  SoSeparator *f05 = new SoSeparator;
  f05 -> ref();

  // umisteni budovy
  SoTransform *transform = new SoTransform;
  transform->translation.setValue(x,y,z);
  f05->addChild(transform);

  // velikost modelu budovy 
  SoScale *scale = new SoScale;
  scale->scaleFactor.setValue(SCALE_X,SCALE_Y,SCALE_Z);
  f05->addChild(scale);

  // rotace modelu budovy 
  SoRotationXYZ *myRotation = new SoRotationXYZ;     
  myRotation -> angle = alfa;
  myRotation -> axis = SoRotationXYZ::Y;
  f05 -> addChild(myRotation);

  // normaly modelu budovy
  SoNormal *myNormals = new SoNormal;
  myNormals -> vector.setValues(0, 10, f05_normal);
  f05 -> addChild(myNormals); 

  // prirazovani normal PER_FACE_INDEXED 
  SoNormalBinding *myNormalBinding = new SoNormalBinding;
  myNormalBinding -> value = SoNormalBinding::PER_FACE_INDEXED;
  f05 -> addChild(myNormalBinding);

  // propojeni bodu modelu budovy 
  SoCoordinate3 *coords = new SoCoordinate3;
  coords->point.setValues(0, sizeof(f05_vertices)/sizeof(float), f05_vertices);
  f05->addChild(coords);

  // 'vyrez textury' (tvar 2D pro krychly potrebujeme ctverec bereme celou textuturu)' 
  SoTextureCoordinate2 *texCoord = new SoTextureCoordinate2;
  texCoord->point.set1Value(0, SbVec2f(0,0));
  texCoord->point.set1Value(1, SbVec2f(1,0));
  texCoord->point.set1Value(2, SbVec2f(0,1));
  texCoord->point.set1Value(3, SbVec2f(1,1));
  f05->addChild(texCoord);

  // front
  SoTexture2 *texture_f05 = new SoTexture2;
  f05 -> addChild(texture_f05);
	switch(texture) {
		case BRICK: texture_f05 -> filename.setValue("Texture/buildings/f05/brick/texture_f05_front.PNG"); break;
		case COLOR: texture_f05 -> filename.setValue("Texture/buildings/f05/color/texture_f05_front.PNG"); break;
		case STONE: texture_f05 -> filename.setValue("Texture/buildings/f05/stone/texture_f05_front.PNG"); break;
		case WOOD:  texture_f05 -> filename.setValue("Texture/buildings/f05/wood/texture_f05_front.PNG");  break;
	}


  SoIndexedTriangleStripSet *stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(f05_indices_front)/sizeof(int32_t), f05_indices_front);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_f05_front)/sizeof(int), normalIndex_f05_front);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_f05_front)/sizeof(int32_t), TexCoordIndex_f05_front); 
  f05->addChild(stripSet);


  // up
  texture_f05 = new SoTexture2;
  f05 -> addChild(texture_f05);
	switch(texture) {
		case BRICK: texture_f05 -> filename.setValue("Texture/buildings/f05/brick/texture_f05_up.PNG"); break;
		case COLOR: texture_f05 -> filename.setValue("Texture/buildings/f05/color/texture_f05_up.PNG"); break;
		case STONE: texture_f05 -> filename.setValue("Texture/buildings/f05/stone/texture_f05_up.PNG"); break;
		case WOOD:  texture_f05 -> filename.setValue("Texture/buildings/f05/wood/texture_f05_up.PNG");  break;
	}


  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(f05_indices_up)/sizeof(int32_t), f05_indices_up);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_f05_up)/sizeof(int), normalIndex_f05_up);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_f05_up)/sizeof(int32_t), TexCoordIndex_f05_up); 
  f05->addChild(stripSet);


  // left
  texture_f05 = new SoTexture2;
  f05 -> addChild(texture_f05);
	switch(texture) {
		case BRICK: texture_f05 -> filename.setValue("Texture/buildings/f05/brick/texture_f05_left.PNG"); break;
		case COLOR: texture_f05 -> filename.setValue("Texture/buildings/f05/color/texture_f05_left.PNG"); break;
		case STONE: texture_f05 -> filename.setValue("Texture/buildings/f05/stone/texture_f05_left.PNG"); break;
		case WOOD:  texture_f05 -> filename.setValue("Texture/buildings/f05/wood/texture_f05_left.PNG");  break;
	}


  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(f05_indices_left)/sizeof(int32_t), f05_indices_left);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_f05_left)/sizeof(int), normalIndex_f05_left);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_f05_left)/sizeof(int32_t), TexCoordIndex_f05_left); 
  f05->addChild(stripSet);


  // right
  texture_f05 = new SoTexture2;
  f05 -> addChild(texture_f05);
	switch(texture) {
		case BRICK: texture_f05 -> filename.setValue("Texture/buildings/f05/brick/texture_f05_right.PNG"); break;
		case COLOR: texture_f05 -> filename.setValue("Texture/buildings/f05/color/texture_f05_right.PNG"); break;
		case STONE: texture_f05 -> filename.setValue("Texture/buildings/f05/stone/texture_f05_right.PNG"); break;
		case WOOD:  texture_f05 -> filename.setValue("Texture/buildings/f05/wood/texture_f05_right.PNG");  break;
	}


  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(f05_indices_right)/sizeof(int32_t), f05_indices_right);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_f05_right)/sizeof(int), normalIndex_f05_right);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_f05_right)/sizeof(int32_t), TexCoordIndex_f05_right); 
  f05->addChild(stripSet);

  
  // back
  texture_f05 = new SoTexture2;
  f05 -> addChild(texture_f05);
	switch(texture) {
		case BRICK: texture_f05 -> filename.setValue("Texture/buildings/f05/brick/texture_f05_back.PNG"); break;
		case COLOR: texture_f05 -> filename.setValue("Texture/buildings/f05/color/texture_f05_back.PNG"); break;
		case STONE: texture_f05 -> filename.setValue("Texture/buildings/f05/stone/texture_f05_back.PNG"); break;
		case WOOD:  texture_f05 -> filename.setValue("Texture/buildings/f05/wood/texture_f05_back.PNG");  break;
	}


  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(f05_indices_back)/sizeof(int32_t), f05_indices_back);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_f05_back)/sizeof(int), normalIndex_f05_back);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_f05_back)/sizeof(int32_t), TexCoordIndex_f05_back); 
  f05->addChild(stripSet);


  // bottom
  texture_f05 = new SoTexture2;
  f05 -> addChild(texture_f05);
  texture_f05 -> filename.setValue("texture_bottom.PNG");

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(f05_indices_bottom)/sizeof(int32_t), f05_indices_bottom);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_f05_bottom)/sizeof(int), normalIndex_f05_bottom);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_f05_bottom)/sizeof(int32_t), TexCoordIndex_f05_bottom); 
  f05->addChild(stripSet);

  f05 -> unrefNoDelete();

  return f05;
} 
// ----------------------------------------------------------------------------- 


// =============================================================================
// konec souboru building_f05.cpp
// =============================================================================