// =============================================================================
// soubor building_f08.cpp
// =============================================================================

#include"..\buildings.h"

// VERTICES 
// body v prostoru jejich propojenim vytvarim model budovy 
static float f08_vertices[][3] = {
	-1, 0, -0.5f,  // 0
	 1, 0, -0.5f,  // 1
  -1, 3, -0.5f,  // 2
   1, 3, -0.5f,  // 3

	-1, 0,  0.5f,  // 4
   1, 0,  0.5f,  // 5
  -1, 3,  0.5f,  // 6
   1, 3,  0.5f   // 7
};

// NORMALS
// normaly pro dobre osvetleni sten modelu budovy
static float f08_normal[][3] = {
	 1,  0,  0, // 0 x 
   0,  1,  0, // 1 y
   0,  0,  1, // 2 z
 
	-1,  0,  0, // 3 -x
   0, -1,  0, // 4 -y
   0,  0, -1, // 5 -z
};

// steny z bodu
static int32_t f08_indices_front[] = {0,1,2,3, -1};
static int32_t f08_indices_left[] = {0,2,4,6, -1};
static int32_t f08_indices_right[] = {1,3,5,7, -1};
static int32_t f08_indices_back[] = {4,5,6,7, -1};
static int32_t f08_indices_up[] = {2,3,6,7, -1};
static int32_t f08_indices_bottom[] = {0,1,4,5, -1};

// normaly ke stenam
static int normalIndex_f08_front[] = {5,5};   // -z     
static int normalIndex_f08_left[] = {3,3};    // -x
static int normalIndex_f08_right[] = {0,0};   //  x
static int normalIndex_f08_back[] = {2,2};    //  z
static int normalIndex_f08_up[] = {1,1};      //  y  
static int normalIndex_f08_bottom[] = {4,4};  // -y

// texturovaci souradnice pro jednotlive steny
static int32_t TexCoordIndex_f08_front[] = {0,1,2,3, -1};
static int32_t TexCoordIndex_f08_up[] = {0,1,2,3, -1};
static int32_t TexCoordIndex_f08_left[] = {0,2,1,3, -1};
static int32_t TexCoordIndex_f08_right[] = {0,2,1,3, -1};
static int32_t TexCoordIndex_f08_back[] = {0,1,2,3, -1};
static int32_t TexCoordIndex_f08_bottom[] = {0,1,2,3, -1};


// -----------------------------------------------------------------------------
// fce.MakeBuilding_f08 - vytvori budovu f08
// ----------------------------------------------------------------------------- 
SoSeparator *MakeBuilding_f08(float x, float y, float z, float alfa, TEXTURE texture) {

  // uzel oznacujici budovu 
  SoSeparator *f08 = new SoSeparator;
  f08 -> ref();

  // umisteni budovy
  SoTransform *transform = new SoTransform;
  transform->translation.setValue(x,y,z);
  f08->addChild(transform);

  // velikost modelu budovy 
  SoScale *scale = new SoScale;
  scale->scaleFactor.setValue(SCALE_X,SCALE_Y,SCALE_Z);
  f08->addChild(scale);

  // rotace modelu budovy 
  SoRotationXYZ *myRotation = new SoRotationXYZ;     
  myRotation -> angle = alfa;
  myRotation -> axis = SoRotationXYZ::Y;
  f08 -> addChild(myRotation);

  // normaly modelu budovy
  SoNormal *myNormals = new SoNormal;
  myNormals -> vector.setValues(0, 10, f08_normal);
  f08 -> addChild(myNormals); 

  // prirazovani normal PER_FACE_INDEXED 
  SoNormalBinding *myNormalBinding = new SoNormalBinding;
  myNormalBinding -> value = SoNormalBinding::PER_FACE_INDEXED;
  f08 -> addChild(myNormalBinding);

  // propojeni bodu modelu budovy 
  SoCoordinate3 *coords = new SoCoordinate3;
  coords->point.setValues(0, sizeof(f08_vertices)/sizeof(float), f08_vertices);
  f08->addChild(coords);

  // 'vyrez textury' (tvar 2D pro krychly potrebujeme ctverec bereme celou textuturu)' 
  SoTextureCoordinate2 *texCoord = new SoTextureCoordinate2;
  texCoord->point.set1Value(0, SbVec2f(0,0));
  texCoord->point.set1Value(1, SbVec2f(1,0));
  texCoord->point.set1Value(2, SbVec2f(0,1));
  texCoord->point.set1Value(3, SbVec2f(1,1));
  f08->addChild(texCoord);

  // front
  SoTexture2 *texture_f08 = new SoTexture2;
  f08 -> addChild(texture_f08);
	switch(texture) {
		case BRICK: texture_f08 -> filename.setValue("Texture/buildings/f08/brick/texture_f08_front.PNG"); break;
		case COLOR: texture_f08 -> filename.setValue("Texture/buildings/f08/color/texture_f08_front.PNG"); break;
		case STONE: texture_f08 -> filename.setValue("Texture/buildings/f08/stone/texture_f08_front.PNG"); break;
		case WOOD:  texture_f08 -> filename.setValue("Texture/buildings/f08/wood/texture_f08_front.PNG");  break;
	}


  SoIndexedTriangleStripSet *stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(f08_indices_front)/sizeof(int32_t), f08_indices_front);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_f08_front)/sizeof(int), normalIndex_f08_front);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_f08_front)/sizeof(int32_t), TexCoordIndex_f08_front); 
  f08->addChild(stripSet);


  // up
  texture_f08 = new SoTexture2;
  f08 -> addChild(texture_f08);
	switch(texture) {
		case BRICK: texture_f08 -> filename.setValue("Texture/buildings/f08/brick/texture_f08_up.PNG"); break;
		case COLOR: texture_f08 -> filename.setValue("Texture/buildings/f08/color/texture_f08_up.PNG"); break;
		case STONE: texture_f08 -> filename.setValue("Texture/buildings/f08/stone/texture_f08_up.PNG"); break;
		case WOOD:  texture_f08 -> filename.setValue("Texture/buildings/f08/wood/texture_f08_up.PNG");  break;
	}

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(f08_indices_up)/sizeof(int32_t), f08_indices_up);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_f08_up)/sizeof(int), normalIndex_f08_up);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_f08_up)/sizeof(int32_t), TexCoordIndex_f08_up); 
  f08->addChild(stripSet);


  // left
  texture_f08 = new SoTexture2;
  f08 -> addChild(texture_f08);
	switch(texture) {
		case BRICK: texture_f08 -> filename.setValue("Texture/buildings/f08/brick/texture_f08_left.PNG"); break;
		case COLOR: texture_f08 -> filename.setValue("Texture/buildings/f08/color/texture_f08_left.PNG"); break;
		case STONE: texture_f08 -> filename.setValue("Texture/buildings/f08/stone/texture_f08_left.PNG"); break;
		case WOOD:  texture_f08 -> filename.setValue("Texture/buildings/f08/wood/texture_f08_left.PNG");  break;
	}

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(f08_indices_left)/sizeof(int32_t), f08_indices_left);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_f08_left)/sizeof(int), normalIndex_f08_left);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_f08_left)/sizeof(int32_t), TexCoordIndex_f08_left); 
  f08->addChild(stripSet);


  // right
  texture_f08 = new SoTexture2;
  f08 -> addChild(texture_f08);
	switch(texture) {
		case BRICK: texture_f08 -> filename.setValue("Texture/buildings/f08/brick/texture_f08_right.PNG"); break;
		case COLOR: texture_f08 -> filename.setValue("Texture/buildings/f08/color/texture_f08_right.PNG"); break;
		case STONE: texture_f08 -> filename.setValue("Texture/buildings/f08/stone/texture_f08_right.PNG"); break;
		case WOOD:  texture_f08 -> filename.setValue("Texture/buildings/f08/wood/texture_f08_right.PNG");  break;
	}

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(f08_indices_right)/sizeof(int32_t), f08_indices_right);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_f08_right)/sizeof(int), normalIndex_f08_right);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_f08_right)/sizeof(int32_t), TexCoordIndex_f08_right); 
  f08->addChild(stripSet);

  
  // back
  texture_f08 = new SoTexture2;
  f08 -> addChild(texture_f08);
	switch(texture) {
		case BRICK: texture_f08 -> filename.setValue("Texture/buildings/f08/brick/texture_f08_back.PNG"); break;
		case COLOR: texture_f08 -> filename.setValue("Texture/buildings/f08/color/texture_f08_back.PNG"); break;
		case STONE: texture_f08 -> filename.setValue("Texture/buildings/f08/stone/texture_f08_back.PNG"); break;
		case WOOD:  texture_f08 -> filename.setValue("Texture/buildings/f08/wood/texture_f08_back.PNG");  break;
	}

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(f08_indices_back)/sizeof(int32_t), f08_indices_back);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_f08_back)/sizeof(int), normalIndex_f08_back);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_f08_back)/sizeof(int32_t), TexCoordIndex_f08_back); 
  f08->addChild(stripSet);


  // bottom
  texture_f08 = new SoTexture2;
  f08 -> addChild(texture_f08);
  texture_f08 -> filename.setValue("texture_bottom.PNG");

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(f08_indices_bottom)/sizeof(int32_t), f08_indices_bottom);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_f08_bottom)/sizeof(int), normalIndex_f08_bottom);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_f08_bottom)/sizeof(int32_t), TexCoordIndex_f08_bottom); 
  f08->addChild(stripSet);

  f08 -> unrefNoDelete();

  return f08;
}
// ----------------------------------------------------------------------------- 


// =============================================================================
// konec souboru building_f08.cpp
// =============================================================================