// =============================================================================
// soubor building_f10.cpp
// =============================================================================

#include"..\buildings.h"

// VERTICES 
// body v prostoru jejich propojenim vytvarim model budovy 
static float f10_vertices[][3] = {
	-1.5f, 0, -1,  // 0
 	 1.5f, 0, -1,  // 1
  -1.5f, 2, -1,  // 2
   1.5f, 2, -1,  // 3

	-1.5f, 0,  0,  // 4
   0.5f, 0,  0,  // 5
  -1.5f, 2,  0,  // 6
   0.5f, 2,  0,  // 7

	 0.5f, 0,  1,  // 8
   1.5f, 0,  1,  // 9
   0.5f, 2,  1,  // 10
   1.5f, 2,  1   // 11
};

// NORMALS
// normaly pro dobre osvetleni sten modelu budovy
static float f10_normal[][3] = {
	 1,  0,  0, // 0 x 
   0,  1,  0, // 1 y
   0,  0,  1, // 2 z
 
	-1,  0,  0, // 3 -x
   0, -1,  0, // 4 -y
   0,  0, -1, // 5 -z
};

// steny z bodu
static int32_t f10_indices_front[] = {0,1,2,3, -1};
static int32_t f10_indices_left[] = {0,2,4,6, -1};
static int32_t f10_indices_right_front[] = {1,3,9,11, -1};
static int32_t f10_indices_right_back[] = {5,7,8,10, -1};
static int32_t f10_indices_back_near[] = {4,5,6,7, -1};
static int32_t f10_indices_back_far[] = {8,9,10,11, -1};
static int32_t f10_indices_up[] = {2,6,3,7,11,10, -1};
static int32_t f10_indices_bottom[] = {0,4,1,5,9,8, -1};

// normaly ke stenam
static int normalIndex_f10_front[] = {5,5};   // -z     
static int normalIndex_f10_left[] = {3,3};    // -x
static int normalIndex_f10_right_front[] = {0,0};   //  x
static int normalIndex_f10_right_back[] = {3,3};   //  x
static int normalIndex_f10_back_near[] = {2,2};    //  z
static int normalIndex_f10_back_far[] = {2,2};    //  z
static int normalIndex_f10_up[] = {1,1,1,1};      //  y  
static int normalIndex_f10_bottom[] = {4,4,4,4};  // -y

// texturovaci souradnice pro jednotlive steny
static int32_t TexCoordIndex_f10_front[] = {0,1,2,3, -1};
static int32_t TexCoordIndex_f10_up[] = {0,4,1,5,3,6, -1};
static int32_t TexCoordIndex_f10_left[] = {0,2,1,3, -1};
static int32_t TexCoordIndex_f10_right_front[] = {0,2,1,3, -1};
static int32_t TexCoordIndex_f10_right_back[] = {0,2,1,3, -1};
static int32_t TexCoordIndex_f10_back_near[] = {0,1,2,3, -1};
static int32_t TexCoordIndex_f10_back_far[] = {0,1,2,3, -1};
static int32_t TexCoordIndex_f10_bottom[] = {0,4,1,5,3,6, -1};


// -----------------------------------------------------------------------------
// fce.MakeBuilding_f10 - vytvori budovu f10
// -----------------------------------------------------------------------------
SoSeparator *MakeBuilding_f10(float x, float y, float z, float alfa, TEXTURE texture) {

  // uzel oznacujici budovu 
  SoSeparator *f10 = new SoSeparator;
  f10 -> ref();

  // umisteni budovy
  SoTransform *transform = new SoTransform;
  transform->translation.setValue(x,y,z);
  f10->addChild(transform);

  // velikost modelu budovy 
  SoScale *scale = new SoScale;
  scale->scaleFactor.setValue(SCALE_X,SCALE_Y,SCALE_Z);
  f10->addChild(scale);

  // rotace modelu budovy 
  SoRotationXYZ *myRotation = new SoRotationXYZ;     
  myRotation -> angle = alfa;
  myRotation -> axis = SoRotationXYZ::Y;
  f10 -> addChild(myRotation);

  // normaly modelu budovy
  SoNormal *myNormals = new SoNormal;
  myNormals -> vector.setValues(0, 10, f10_normal);
  f10 -> addChild(myNormals); 

  // prirazovani normal PER_FACE_INDEXED 
  SoNormalBinding *myNormalBinding = new SoNormalBinding;
  myNormalBinding -> value = SoNormalBinding::PER_FACE_INDEXED;
  f10 -> addChild(myNormalBinding);

  // propojeni bodu modelu budovy 
  SoCoordinate3 *coords = new SoCoordinate3;
  coords->point.setValues(0, sizeof(f10_vertices)/sizeof(float), f10_vertices);
  f10->addChild(coords);

  // 'vyrez textury' (tvar 2D pro krychly potrebujeme ctverec bereme celou textuturu)' 
  SoTextureCoordinate2 *texCoord = new SoTextureCoordinate2;
  texCoord->point.set1Value(0, SbVec2f(0,0));
  texCoord->point.set1Value(1, SbVec2f(1,0));
  texCoord->point.set1Value(2, SbVec2f(0,1));
  texCoord->point.set1Value(3, SbVec2f(1,1));

  texCoord->point.set1Value(4, SbVec2f(0,0.5));
  texCoord->point.set1Value(5, SbVec2f(0.6666f,0.5));
  texCoord->point.set1Value(6, SbVec2f(0.6666f,1));
  f10->addChild(texCoord);

  // front
  SoTexture2 *texture_f10 = new SoTexture2;
  f10 -> addChild(texture_f10);
	switch(texture) {
		case BRICK: texture_f10 -> filename.setValue("Texture/buildings/f10/brick/texture_f10_front.PNG"); break;
		case COLOR: texture_f10 -> filename.setValue("Texture/buildings/f10/color/texture_f10_front.PNG"); break;
		case STONE: texture_f10 -> filename.setValue("Texture/buildings/f10/stone/texture_f10_front.PNG"); break;
		case WOOD:  texture_f10 -> filename.setValue("Texture/buildings/f10/wood/texture_f10_front.PNG");  break;
	}

  SoIndexedTriangleStripSet *stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(f10_indices_front)/sizeof(int32_t), f10_indices_front);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_f10_front)/sizeof(int), normalIndex_f10_front);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_f10_front)/sizeof(int32_t), TexCoordIndex_f10_front); 
  f10->addChild(stripSet);


  // up
  texture_f10 = new SoTexture2;
  f10 -> addChild(texture_f10);
	switch(texture) {
		case BRICK: texture_f10 -> filename.setValue("Texture/buildings/f10/brick/texture_f10_up.PNG"); break;
		case COLOR: texture_f10 -> filename.setValue("Texture/buildings/f10/color/texture_f10_up.PNG"); break;
		case STONE: texture_f10 -> filename.setValue("Texture/buildings/f10/stone/texture_f10_up.PNG"); break;
		case WOOD:  texture_f10 -> filename.setValue("Texture/buildings/f10/wood/texture_f10_up.PNG");  break;
	}

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(f10_indices_up)/sizeof(int32_t), f10_indices_up);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_f10_up)/sizeof(int), normalIndex_f10_up);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_f10_up)/sizeof(int32_t), TexCoordIndex_f10_up); 
  f10->addChild(stripSet);


  // left
  texture_f10 = new SoTexture2;
  f10 -> addChild(texture_f10);
	switch(texture) {
		case BRICK: texture_f10 -> filename.setValue("Texture/buildings/f10/brick/texture_f10_left.PNG"); break;
		case COLOR: texture_f10 -> filename.setValue("Texture/buildings/f10/color/texture_f10_left.PNG"); break;
		case STONE: texture_f10 -> filename.setValue("Texture/buildings/f10/stone/texture_f10_left.PNG"); break;
		case WOOD:  texture_f10 -> filename.setValue("Texture/buildings/f10/wood/texture_f10_left.PNG");  break;
	}

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(f10_indices_left)/sizeof(int32_t), f10_indices_left);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_f10_left)/sizeof(int), normalIndex_f10_left);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_f10_left)/sizeof(int32_t), TexCoordIndex_f10_left); 
  f10->addChild(stripSet);


  // right_front
  texture_f10 = new SoTexture2;
  f10 -> addChild(texture_f10);
	switch(texture) {
		case BRICK: texture_f10 -> filename.setValue("Texture/buildings/f10/brick/texture_f10_right_front.PNG"); break;
		case COLOR: texture_f10 -> filename.setValue("Texture/buildings/f10/color/texture_f10_right_front.PNG"); break;
		case STONE: texture_f10 -> filename.setValue("Texture/buildings/f10/stone/texture_f10_right_front.PNG"); break;
		case WOOD:  texture_f10 -> filename.setValue("Texture/buildings/f10/wood/texture_f10_right_front.PNG");  break;
	}

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(f10_indices_right_front)/sizeof(int32_t), f10_indices_right_front);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_f10_right_front)/sizeof(int), normalIndex_f10_right_front);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_f10_right_front)/sizeof(int32_t), TexCoordIndex_f10_right_front); 
  f10->addChild(stripSet);

  // right_back
  texture_f10 = new SoTexture2;
  f10 -> addChild(texture_f10);
	switch(texture) {
		case BRICK: texture_f10 -> filename.setValue("Texture/buildings/f10/brick/texture_f10_right_back.PNG"); break;
		case COLOR: texture_f10 -> filename.setValue("Texture/buildings/f10/color/texture_f10_right_back.PNG"); break;
		case STONE: texture_f10 -> filename.setValue("Texture/buildings/f10/stone/texture_f10_right_back.PNG"); break;
		case WOOD:  texture_f10 -> filename.setValue("Texture/buildings/f10/wood/texture_f10_right_back.PNG");  break;
	}

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(f10_indices_right_back)/sizeof(int32_t), f10_indices_right_back);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_f10_right_back)/sizeof(int), normalIndex_f10_right_back);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_f10_right_back)/sizeof(int32_t), TexCoordIndex_f10_right_back); 
  f10->addChild(stripSet);

  
  // back_near
  texture_f10 = new SoTexture2;
  f10 -> addChild(texture_f10);
	switch(texture) {
		case BRICK: texture_f10 -> filename.setValue("Texture/buildings/f10/brick/texture_f10_back_near.PNG"); break;
		case COLOR: texture_f10 -> filename.setValue("Texture/buildings/f10/color/texture_f10_back_near.PNG"); break;
		case STONE: texture_f10 -> filename.setValue("Texture/buildings/f10/stone/texture_f10_back_near.PNG"); break;
		case WOOD:  texture_f10 -> filename.setValue("Texture/buildings/f10/wood/texture_f10_back_near.PNG");  break;
	}

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(f10_indices_back_near)/sizeof(int32_t), f10_indices_back_near);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_f10_back_near)/sizeof(int), normalIndex_f10_back_near);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_f10_back_near)/sizeof(int32_t), TexCoordIndex_f10_back_near); 
  f10->addChild(stripSet);

  // back_far
  texture_f10 = new SoTexture2;
  f10 -> addChild(texture_f10);
	switch(texture) {
		case BRICK: texture_f10 -> filename.setValue("Texture/buildings/f10/brick/texture_f10_back_far.PNG"); break;
		case COLOR: texture_f10 -> filename.setValue("Texture/buildings/f10/color/texture_f10_back_far.PNG"); break;
		case STONE: texture_f10 -> filename.setValue("Texture/buildings/f10/stone/texture_f10_back_far.PNG"); break;
		case WOOD:  texture_f10 -> filename.setValue("Texture/buildings/f10/wood/texture_f10_back_far.PNG");  break;
	}

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(f10_indices_back_far)/sizeof(int32_t), f10_indices_back_far);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_f10_back_far)/sizeof(int), normalIndex_f10_back_far);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_f10_back_far)/sizeof(int32_t), TexCoordIndex_f10_back_far); 
  f10->addChild(stripSet);


  // bottom
  texture_f10 = new SoTexture2;
  f10 -> addChild(texture_f10);
  texture_f10 -> filename.setValue("texture_bottom.PNG");

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(f10_indices_bottom)/sizeof(int32_t), f10_indices_bottom);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_f10_bottom)/sizeof(int), normalIndex_f10_bottom);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_f10_bottom)/sizeof(int32_t), TexCoordIndex_f10_bottom); 
  f10->addChild(stripSet);


  f10 -> unrefNoDelete();

  return f10;
}
// ----------------------------------------------------------------------------- 


// =============================================================================
// konec souboru building_f10.cpp
// =============================================================================