// =============================================================================
// soubor building_f11.cpp
// =============================================================================

#include"..\buildings.h"

// VERTICES 
// body v prostoru jejich propojenim vytvarim model budovy 
static float f11_vertices[][3] = {
	-2.5f, 0, -1,  // 0
	 2.5f, 0, -1,  // 1
  -2.5f, 3, -1,  // 2
      1, 3, -1,  // 3
      1, 2, -1,  // 4
   2.5f, 2, -1,  // 5

	-2.5f, 0,  1,  // 6
   2.5f, 0,  1,  // 7
  -2.5f, 3,  1,  // 8
      1, 3,  1,  // 9
      1, 2,  1,  // 10
   2.5f, 2,  1,  // 11
};

// NORMALS
// normaly pro dobre osvetleni sten modelu budovy
static float f11_normal[][3] = {
	 1,  0,  0, // 0 x 
   0,  1,  0, // 1 y
   0,  0,  1, // 2 z
 
	-1,  0,  0, // 3 -x
   0, -1,  0, // 4 -y
   0,  0, -1, // 5 -z
};

// steny z bodu
static int32_t f11_indices_front[] = {2,3,0,4,1,5, -1};
static int32_t f11_indices_left[] = {0,2,6,8, -1};
static int32_t f11_indices_right_near[] = {1,5,7,11, -1};
static int32_t f11_indices_right_far[] = {3,4,9,10, -1};
static int32_t f11_indices_back[] = {8,9,6,10,7,11, -1};
static int32_t f11_indices_up_high[] = {2,3,8,9, -1};
static int32_t f11_indices_up_low[] = {4,5,10,11, -1};
static int32_t f11_indices_bottom[] = {0,1,6,7, -1};

// normaly ke stenam
static int normalIndex_f11_front[] = {5,5,5,5};   // -z     
static int normalIndex_f11_left[] = {3,3};    // -x
static int normalIndex_f11_right_near[] = {0,0};   //  x
static int normalIndex_f11_right_far[] = {0,0};   //  x
static int normalIndex_f11_back[] = {2,2,2,2};    //  z
static int normalIndex_f11_up_high[] = {1,1};      //  y 
static int normalIndex_f11_up_low[] = {1,1};      //  y 
static int normalIndex_f11_bottom[] = {4,4};  // -y

// texturovaci souradnice pro jednotlive steny
static int32_t TexCoordIndex_f11_front[] = {2,4,0,5,1,6, -1};
static int32_t TexCoordIndex_f11_up_high[] = {0,1,2,3, -1};
static int32_t TexCoordIndex_f11_up_low[] = {0,1,2,3, -1};
static int32_t TexCoordIndex_f11_left[] = {0,2,1,3, -1};
static int32_t TexCoordIndex_f11_right_near[] = {0,2,1,3, -1};
static int32_t TexCoordIndex_f11_right_far[] = {0,2,1,3, -1};
static int32_t TexCoordIndex_f11_back[] = {2,4,0,5,1,6, -1};
static int32_t TexCoordIndex_f11_bottom[] = {0,1,2,3, -1};


// -----------------------------------------------------------------------------
// fce.MakeBuilding_f11 - vytvori budovu f11
// -----------------------------------------------------------------------------
SoSeparator *MakeBuilding_f11(float x, float y, float z, float alfa, TEXTURE texture) {

  // uzel oznacujici budovu 
  SoSeparator *f11 = new SoSeparator;
  f11 -> ref();

  // umisteni budovy
  SoTransform *transform = new SoTransform;
  transform->translation.setValue(x,y,z);
  f11->addChild(transform);

  // velikost modelu budovy 
  SoScale *scale = new SoScale;
  scale->scaleFactor.setValue(SCALE_X,SCALE_Y,SCALE_Z);
  f11->addChild(scale);

  // rotace modelu budovy 
  SoRotationXYZ *myRotation = new SoRotationXYZ;     
  myRotation -> angle = alfa;
  myRotation -> axis = SoRotationXYZ::Y;
  f11 -> addChild(myRotation);

  // normaly modelu budovy
  SoNormal *myNormals = new SoNormal;
  myNormals -> vector.setValues(0, 10, f11_normal);
  f11 -> addChild(myNormals); 

  // prirazovani normal PER_FACE_INDEXED 
  SoNormalBinding *myNormalBinding = new SoNormalBinding;
  myNormalBinding -> value = SoNormalBinding::PER_FACE_INDEXED;
  f11 -> addChild(myNormalBinding);

  // propojeni bodu modelu budovy 
  SoCoordinate3 *coords = new SoCoordinate3;
  coords->point.setValues(0, sizeof(f11_vertices)/sizeof(float), f11_vertices);
  f11->addChild(coords);

  // 'vyrez textury' (tvar 2D pro krychly potrebujeme ctverec bereme celou textuturu)' 
  SoTextureCoordinate2 *texCoord = new SoTextureCoordinate2;
  texCoord->point.set1Value(0, SbVec2f(0,0));
  texCoord->point.set1Value(1, SbVec2f(1,0));
  texCoord->point.set1Value(2, SbVec2f(0,1));
  texCoord->point.set1Value(3, SbVec2f(1,1));

  texCoord->point.set1Value(4, SbVec2f(0.7f,1)); // 3 9
  texCoord->point.set1Value(5, SbVec2f(0.7f,0.6666f)); // 4 10
  texCoord->point.set1Value(6, SbVec2f(1,0.6666f)); // 5 11
  f11->addChild(texCoord);

  // front
  SoTexture2 *texture_f11 = new SoTexture2;
  f11 -> addChild(texture_f11);
	switch(texture) {
		case BRICK: texture_f11 -> filename.setValue("Texture/buildings/f11/brick/texture_f11_front.PNG"); break;
		case COLOR: texture_f11 -> filename.setValue("Texture/buildings/f11/color/texture_f11_front.PNG"); break;
		case STONE: texture_f11 -> filename.setValue("Texture/buildings/f11/stone/texture_f11_front.PNG"); break;
		case WOOD:  texture_f11 -> filename.setValue("Texture/buildings/f11/wood/texture_f11_front.PNG");  break;
	}

  SoIndexedTriangleStripSet *stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(f11_indices_front)/sizeof(int32_t), f11_indices_front);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_f11_front)/sizeof(int), normalIndex_f11_front);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_f11_front)/sizeof(int32_t), TexCoordIndex_f11_front); 
  f11->addChild(stripSet);


  // up_high
  texture_f11 = new SoTexture2;
  f11 -> addChild(texture_f11);
	switch(texture) {
		case BRICK: texture_f11 -> filename.setValue("Texture/buildings/f11/brick/texture_f11_up_high.PNG"); break;
		case COLOR: texture_f11 -> filename.setValue("Texture/buildings/f11/color/texture_f11_up_high.PNG"); break;
		case STONE: texture_f11 -> filename.setValue("Texture/buildings/f11/stone/texture_f11_up_high.PNG"); break;
		case WOOD:  texture_f11 -> filename.setValue("Texture/buildings/f11/wood/texture_f11_up_high.PNG");  break;
	}

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(f11_indices_up_high)/sizeof(int32_t), f11_indices_up_high);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_f11_up_high)/sizeof(int), normalIndex_f11_up_high);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_f11_up_high)/sizeof(int32_t), TexCoordIndex_f11_up_high); 
  f11->addChild(stripSet);

  // up_low
  texture_f11 = new SoTexture2;
  f11 -> addChild(texture_f11);
	switch(texture) {
		case BRICK: texture_f11 -> filename.setValue("Texture/buildings/f11/brick/texture_f11_up_low.PNG"); break;
		case COLOR: texture_f11 -> filename.setValue("Texture/buildings/f11/color/texture_f11_up_low.PNG"); break;
		case STONE: texture_f11 -> filename.setValue("Texture/buildings/f11/stone/texture_f11_up_low.PNG"); break;
		case WOOD:  texture_f11 -> filename.setValue("Texture/buildings/f11/wood/texture_f11_up_low.PNG");  break;
	}

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(f11_indices_up_low)/sizeof(int32_t), f11_indices_up_low);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_f11_up_low)/sizeof(int), normalIndex_f11_up_low);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_f11_up_low)/sizeof(int32_t), TexCoordIndex_f11_up_low); 
  f11->addChild(stripSet);


  // left
  texture_f11 = new SoTexture2;
  f11 -> addChild(texture_f11);
	switch(texture) {
		case BRICK: texture_f11 -> filename.setValue("Texture/buildings/f11/brick/texture_f11_left.PNG"); break;
		case COLOR: texture_f11 -> filename.setValue("Texture/buildings/f11/color/texture_f11_left.PNG"); break;
		case STONE: texture_f11 -> filename.setValue("Texture/buildings/f11/stone/texture_f11_left.PNG"); break;
		case WOOD:  texture_f11 -> filename.setValue("Texture/buildings/f11/wood/texture_f11_left.PNG");  break;
	}

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(f11_indices_left)/sizeof(int32_t), f11_indices_left);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_f11_left)/sizeof(int), normalIndex_f11_left);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_f11_left)/sizeof(int32_t), TexCoordIndex_f11_left); 
  f11->addChild(stripSet);


  // right_near
  texture_f11 = new SoTexture2;
  f11 -> addChild(texture_f11);
	switch(texture) {
		case BRICK: texture_f11 -> filename.setValue("Texture/buildings/f11/brick/texture_f11_right_near.PNG"); break;
		case COLOR: texture_f11 -> filename.setValue("Texture/buildings/f11/color/texture_f11_right_near.PNG"); break;
		case STONE: texture_f11 -> filename.setValue("Texture/buildings/f11/stone/texture_f11_right_near.PNG"); break;
		case WOOD:  texture_f11 -> filename.setValue("Texture/buildings/f11/wood/texture_f11_right_near.PNG");  break;
	}

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(f11_indices_right_near)/sizeof(int32_t), f11_indices_right_near);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_f11_right_near)/sizeof(int), normalIndex_f11_right_near);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_f11_right_near)/sizeof(int32_t), TexCoordIndex_f11_right_near); 
  f11->addChild(stripSet);

  // right_far
  texture_f11 = new SoTexture2;
  f11 -> addChild(texture_f11);
	switch(texture) {
		case BRICK: texture_f11 -> filename.setValue("Texture/buildings/f11/brick/texture_f11_right_far.PNG"); break;
		case COLOR: texture_f11 -> filename.setValue("Texture/buildings/f11/color/texture_f11_right_far.PNG"); break;
		case STONE: texture_f11 -> filename.setValue("Texture/buildings/f11/stone/texture_f11_right_far.PNG"); break;
		case WOOD:  texture_f11 -> filename.setValue("Texture/buildings/f11/wood/texture_f11_right_far.PNG");  break;
	}

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(f11_indices_right_far)/sizeof(int32_t), f11_indices_right_far);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_f11_right_far)/sizeof(int), normalIndex_f11_right_far);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_f11_right_far)/sizeof(int32_t), TexCoordIndex_f11_right_far); 
  f11->addChild(stripSet);

  
  // back
  texture_f11 = new SoTexture2;
  f11 -> addChild(texture_f11);
	switch(texture) {
		case BRICK: texture_f11 -> filename.setValue("Texture/buildings/f11/brick/texture_f11_back.PNG"); break;
		case COLOR: texture_f11 -> filename.setValue("Texture/buildings/f11/color/texture_f11_back.PNG"); break;
		case STONE: texture_f11 -> filename.setValue("Texture/buildings/f11/stone/texture_f11_back.PNG"); break;
		case WOOD:  texture_f11 -> filename.setValue("Texture/buildings/f11/wood/texture_f11_back.PNG");  break;
	}

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(f11_indices_back)/sizeof(int32_t), f11_indices_back);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_f11_back)/sizeof(int), normalIndex_f11_back);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_f11_back)/sizeof(int32_t), TexCoordIndex_f11_back); 
  f11->addChild(stripSet);


  // bottom
  texture_f11 = new SoTexture2;
  f11 -> addChild(texture_f11);
  texture_f11 -> filename.setValue("texture_bottom.PNG");

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(f11_indices_bottom)/sizeof(int32_t), f11_indices_bottom);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_f11_bottom)/sizeof(int), normalIndex_f11_bottom);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_f11_bottom)/sizeof(int32_t), TexCoordIndex_f11_bottom); 
  f11->addChild(stripSet);

  f11 -> unrefNoDelete();

  return f11;
}
// ----------------------------------------------------------------------------- 


// =============================================================================
// konec souboru building_f11.cpp
// =============================================================================