// =============================================================================
// soubor building_f12.cpp
// =============================================================================

#include"..\buildings.h"

// VERTICES 
// body v prostoru jejich propojenim vytvarim model budovy 
static float f12_vertices[][3] = {
	-2, 0, -1.5f,  // 0
	 2, 0, -1.5f,  // 1
  -2, 2, -1.5f,  // 2
   2, 2, -1.5f,  // 3

	-1, 0, -0.5f,  // 4
   1, 0, -0.5f,  // 5
  -1, 2, -0.5f,  // 6
   1, 2, -0.5f,  // 7

	-2, 0,  1.5f,  // 8
	-1, 0,  1.5f,  // 9
  -2, 2,  1.5f,  // 10
  -1, 2,  1.5f,  // 11

	 1, 0,  1.5f,  // 12
   2, 0,  1.5f,  // 13
   1, 2,  1.5f,  // 14
   2, 2,  1.5f,  // 15
};

// NORMALS
// normaly pro dobre osvetleni sten modelu budovy
static float f12_normal[][3] = {
	 1,  0,  0, // 0 x 
   0,  1,  0, // 1 y
   0,  0,  1, // 2 z
 
	-1,  0,  0, // 3 -x
   0, -1,  0, // 4 -y
   0,  0, -1, // 5 -z
};

// steny z bodu
static int32_t f12_indices_front[] = {0,1,2,3, -1};
static int32_t f12_indices_left_near[] = {0,2,8,10, -1};
static int32_t f12_indices_left_far[] = {5,7,12,14, -1};
static int32_t f12_indices_right_near[] = {1,3,13,15, -1};
static int32_t f12_indices_right_far[] = {4,6,9,11, -1};
static int32_t f12_indices_back_middle[] = {4,5,6,7, -1};
static int32_t f12_indices_back_left[] = {8,9,10,11, -1};
static int32_t f12_indices_back_right[] = {12,13,14,15, -1};
static int32_t f12_indices_up[] = {10,11,2,6,3,7,15,14, -1};
static int32_t f12_indices_bottom[] = {8,9,0,4,1,5,13,12, -1};

// normaly ke stenam
static int normalIndex_f12_front[] = {5,5};   // -z     
static int normalIndex_f12_left_near[] = {3,3};    // -x
static int normalIndex_f12_left_far[] = {3,3};    // -x
static int normalIndex_f12_right_near[] = {0,0};   //  x
static int normalIndex_f12_right_far[] = {0,0};   //  x
static int normalIndex_f12_back_middle[] = {2,2};    //  z
static int normalIndex_f12_back_left[] = {2,2};    //  z
static int normalIndex_f12_back_right[] = {2,2};    //  z
static int normalIndex_f12_up[] = {1,1,1,1,1,1};      //  y  
static int normalIndex_f12_bottom[] = {4,4,4,4,4,4};  // -y

// texturovaci souradnice pro jednotlive steny
static int32_t TexCoordIndex_f12_front[] = {0,1,2,3, -1};
static int32_t TexCoordIndex_f12_up[] = {2,4,0,5,1,6,3,7, -1};
static int32_t TexCoordIndex_f12_left_near[] = {0,2,1,3, -1};
static int32_t TexCoordIndex_f12_left_far[] = {0,2,1,3, -1};
static int32_t TexCoordIndex_f12_right_near[] = {0,2,1,3, -1};
static int32_t TexCoordIndex_f12_right_far[] = {0,2,1,3, -1};
static int32_t TexCoordIndex_f12_back_middle[] = {0,1,2,3, -1};
static int32_t TexCoordIndex_f12_back_left[] = {0,1,2,3, -1};
static int32_t TexCoordIndex_f12_back_right[] = {0,1,2,3, -1};
static int32_t TexCoordIndex_f12_bottom[] = {2,4,0,5,1,6,3,7, -1};


// -----------------------------------------------------------------------------
// fce.MakeBuilding_f12 - vytvori budovu f12
// ----------------------------------------------------------------------------- 
SoSeparator *MakeBuilding_f12(float x, float y, float z, float alfa, TEXTURE texture) {

  // uzel oznacujici budovu 
  SoSeparator *f12 = new SoSeparator;
  f12 -> ref();

  // umisteni budovy
  SoTransform *transform = new SoTransform;
  transform->translation.setValue(x,y,z);
  f12->addChild(transform);

  // velikost modelu budovy 
  SoScale *scale = new SoScale;
  scale->scaleFactor.setValue(SCALE_X,SCALE_Y,SCALE_Z);
  f12->addChild(scale);

  // rotace modelu budovy 
  SoRotationXYZ *myRotation = new SoRotationXYZ;     
  myRotation -> angle = alfa;
  myRotation -> axis = SoRotationXYZ::Y;
  f12 -> addChild(myRotation);

  // normaly modelu budovy
  SoNormal *myNormals = new SoNormal;
  myNormals -> vector.setValues(0, 10, f12_normal);
  f12 -> addChild(myNormals); 

  // prirazovani normal PER_FACE_INDEXED
  SoNormalBinding *myNormalBinding = new SoNormalBinding;
  myNormalBinding -> value = SoNormalBinding::PER_FACE_INDEXED;
  f12 -> addChild(myNormalBinding);

  // propojeni bodu modelu budovy 
  SoCoordinate3 *coords = new SoCoordinate3;
  coords->point.setValues(0, sizeof(f12_vertices)/sizeof(float), f12_vertices);
  f12->addChild(coords);

  // 'vyrez textury' (tvar 2D pro krychly potrebujeme ctverec bereme celou textuturu)' 
  SoTextureCoordinate2 *texCoord = new SoTextureCoordinate2;
  texCoord->point.set1Value(0, SbVec2f(0,0));
  texCoord->point.set1Value(1, SbVec2f(1,0));
  texCoord->point.set1Value(2, SbVec2f(0,1));
  texCoord->point.set1Value(3, SbVec2f(1,1));

  texCoord->point.set1Value(4, SbVec2f(0.25,1));
  texCoord->point.set1Value(5, SbVec2f(0.25,0.3333f));
  texCoord->point.set1Value(6, SbVec2f(0.75,0.3333f));
  texCoord->point.set1Value(7, SbVec2f(0.75,1));
  f12->addChild(texCoord);

  // front
  SoTexture2 *texture_f12 = new SoTexture2;
  f12 -> addChild(texture_f12);
	switch(texture) {
		case BRICK: texture_f12 -> filename.setValue("Texture/buildings/f12/brick/texture_f12_front.PNG"); break;
		case COLOR: texture_f12 -> filename.setValue("Texture/buildings/f12/color/texture_f12_front.PNG"); break;
		case STONE: texture_f12 -> filename.setValue("Texture/buildings/f12/stone/texture_f12_front.PNG"); break;
		case WOOD:  texture_f12 -> filename.setValue("Texture/buildings/f12/wood/texture_f12_front.PNG");  break;
	}

  SoIndexedTriangleStripSet *stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(f12_indices_front)/sizeof(int32_t), f12_indices_front);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_f12_front)/sizeof(int), normalIndex_f12_front);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_f12_front)/sizeof(int32_t), TexCoordIndex_f12_front); 
  f12->addChild(stripSet);


  // up
  texture_f12 = new SoTexture2;
  f12 -> addChild(texture_f12);
	switch(texture) {
		case BRICK: texture_f12 -> filename.setValue("Texture/buildings/f12/brick/texture_f12_up.PNG"); break;
		case COLOR: texture_f12 -> filename.setValue("Texture/buildings/f12/color/texture_f12_up.PNG"); break;
		case STONE: texture_f12 -> filename.setValue("Texture/buildings/f12/stone/texture_f12_up.PNG"); break;
		case WOOD:  texture_f12 -> filename.setValue("Texture/buildings/f12/wood/texture_f12_up.PNG");  break;
	}

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(f12_indices_up)/sizeof(int32_t), f12_indices_up);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_f12_up)/sizeof(int), normalIndex_f12_up);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_f12_up)/sizeof(int32_t), TexCoordIndex_f12_up); 
  f12->addChild(stripSet);


  // left_near
  texture_f12 = new SoTexture2;
  f12 -> addChild(texture_f12);
	switch(texture) {
		case BRICK: texture_f12 -> filename.setValue("Texture/buildings/f12/brick/texture_f12_out.PNG"); break;
		case COLOR: texture_f12 -> filename.setValue("Texture/buildings/f12/color/texture_f12_out.PNG"); break;
		case STONE: texture_f12 -> filename.setValue("Texture/buildings/f12/stone/texture_f12_out.PNG"); break;
		case WOOD:  texture_f12 -> filename.setValue("Texture/buildings/f12/wood/texture_f12_out.PNG");  break;
	}

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(f12_indices_left_near)/sizeof(int32_t), f12_indices_left_near);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_f12_left_near)/sizeof(int), normalIndex_f12_left_near);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_f12_left_near)/sizeof(int32_t), TexCoordIndex_f12_left_near); 
  f12->addChild(stripSet);


  // left_far
  texture_f12 = new SoTexture2;
  f12 -> addChild(texture_f12);
	switch(texture) {
		case BRICK: texture_f12 -> filename.setValue("Texture/buildings/f12/brick/texture_f12_in.PNG"); break;
		case COLOR: texture_f12 -> filename.setValue("Texture/buildings/f12/color/texture_f12_in.PNG"); break;
		case STONE: texture_f12 -> filename.setValue("Texture/buildings/f12/stone/texture_f12_in.PNG"); break;
		case WOOD:  texture_f12 -> filename.setValue("Texture/buildings/f12/wood/texture_f12_in.PNG");  break;
	}

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(f12_indices_left_far)/sizeof(int32_t), f12_indices_left_far);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_f12_left_far)/sizeof(int), normalIndex_f12_left_far);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_f12_left_far)/sizeof(int32_t), TexCoordIndex_f12_left_far); 
  f12->addChild(stripSet);


  // right_near
  texture_f12 = new SoTexture2;
  f12 -> addChild(texture_f12);
	switch(texture) {
		case BRICK: texture_f12 -> filename.setValue("Texture/buildings/f12/brick/texture_f12_in.PNG"); break;
		case COLOR: texture_f12 -> filename.setValue("Texture/buildings/f12/color/texture_f12_in.PNG"); break;
		case STONE: texture_f12 -> filename.setValue("Texture/buildings/f12/stone/texture_f12_in.PNG"); break;
		case WOOD:  texture_f12 -> filename.setValue("Texture/buildings/f12/wood/texture_f12_in.PNG");  break;
	}

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(f12_indices_right_near)/sizeof(int32_t), f12_indices_right_near);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_f12_right_near)/sizeof(int), normalIndex_f12_right_near);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_f12_right_near)/sizeof(int32_t), TexCoordIndex_f12_right_near); 
  f12->addChild(stripSet);


  // right_far
  texture_f12 = new SoTexture2;
  f12 -> addChild(texture_f12);
	switch(texture) {
		case BRICK: texture_f12 -> filename.setValue("Texture/buildings/f12/brick/texture_f12_out.PNG"); break;
		case COLOR: texture_f12 -> filename.setValue("Texture/buildings/f12/color/texture_f12_out.PNG"); break;
		case STONE: texture_f12 -> filename.setValue("Texture/buildings/f12/stone/texture_f12_out.PNG"); break;
		case WOOD:  texture_f12 -> filename.setValue("Texture/buildings/f12/wood/texture_f12_out.PNG");  break;
	}

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(f12_indices_right_far)/sizeof(int32_t), f12_indices_right_far);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_f12_right_far)/sizeof(int), normalIndex_f12_right_far);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_f12_right_far)/sizeof(int32_t), TexCoordIndex_f12_right_far); 
  f12->addChild(stripSet);
  
  // back_middle
  texture_f12 = new SoTexture2;
  f12 -> addChild(texture_f12);
	switch(texture) {
		case BRICK: texture_f12 -> filename.setValue("Texture/buildings/f12/brick/texture_f12_in.PNG"); break;
		case COLOR: texture_f12 -> filename.setValue("Texture/buildings/f12/color/texture_f12_in.PNG"); break;
		case STONE: texture_f12 -> filename.setValue("Texture/buildings/f12/stone/texture_f12_in.PNG"); break;
		case WOOD:  texture_f12 -> filename.setValue("Texture/buildings/f12/wood/texture_f12_in.PNG");  break;
	}

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(f12_indices_back_middle)/sizeof(int32_t), f12_indices_back_middle);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_f12_back_middle)/sizeof(int), normalIndex_f12_back_middle);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_f12_back_middle)/sizeof(int32_t), TexCoordIndex_f12_back_middle); 
  f12->addChild(stripSet);


  // back_left
  texture_f12 = new SoTexture2;
  f12 -> addChild(texture_f12);
	switch(texture) {
		case BRICK: texture_f12 -> filename.setValue("Texture/buildings/f12/brick/texture_f12_back.PNG"); break;
		case COLOR: texture_f12 -> filename.setValue("Texture/buildings/f12/color/texture_f12_back.PNG"); break;
		case STONE: texture_f12 -> filename.setValue("Texture/buildings/f12/stone/texture_f12_back.PNG"); break;
		case WOOD:  texture_f12 -> filename.setValue("Texture/buildings/f12/wood/texture_f12_back.PNG");  break;
	}

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(f12_indices_back_left)/sizeof(int32_t), f12_indices_back_left);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_f12_back_left)/sizeof(int), normalIndex_f12_back_left);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_f12_back_left)/sizeof(int32_t), TexCoordIndex_f12_back_left); 
  f12->addChild(stripSet);


  // back_right
  texture_f12 = new SoTexture2;
  f12 -> addChild(texture_f12);
	switch(texture) {
		case BRICK: texture_f12 -> filename.setValue("Texture/buildings/f12/brick/texture_f12_back.PNG"); break;
		case COLOR: texture_f12 -> filename.setValue("Texture/buildings/f12/color/texture_f12_back.PNG"); break;
		case STONE: texture_f12 -> filename.setValue("Texture/buildings/f12/stone/texture_f12_back.PNG"); break;
		case WOOD:  texture_f12 -> filename.setValue("Texture/buildings/f12/wood/texture_f12_back.PNG");  break;
	}

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(f12_indices_back_right)/sizeof(int32_t), f12_indices_back_right);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_f12_back_right)/sizeof(int), normalIndex_f12_back_right);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_f12_back_right)/sizeof(int32_t), TexCoordIndex_f12_back_right); 
  f12->addChild(stripSet);


  // bottom
  texture_f12 = new SoTexture2;
  f12 -> addChild(texture_f12);
  texture_f12 -> filename.setValue("texture_bottom.PNG");

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(f12_indices_bottom)/sizeof(int32_t), f12_indices_bottom);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_f12_bottom)/sizeof(int), normalIndex_f12_bottom);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_f12_bottom)/sizeof(int32_t), TexCoordIndex_f12_bottom); 
  f12->addChild(stripSet);

  f12 -> unrefNoDelete();

  return f12;
}
// ----------------------------------------------------------------------------- 


// =============================================================================
// konec souboru building_f12.cpp
// =============================================================================