// =============================================================================
// soubor building_p02.cpp
// =============================================================================

#include"..\buildings.h"

// VERTICES 
// body v prostoru jejich propojenim vytvarim model budovy
static float p02_vertices[][3] = {
	-0.5f, 0, -0.5f,  // 0
	 0.5f, 0, -0.5f,  // 1
   0.5f, 2, -0.5f,  // 2
  -0.5f, 2, -0.5f,  // 3

      0, 2.5, 0,  // 4

	 0.5f, 2, 0.5f,  // 5
  -0.5f, 2, 0.5f,  // 6
  -0.5f, 0, 0.5f,  // 7
   0.5f, 0, 0.5f,  // 8
};

// NORMALS
// normaly pro dobre osvetleni sten modelu budovy
static float p02_normal[][3] = {
	  1,  0,  0, // 0 x 
    0,  1,  0, // 1 y
    0,  0,  1, // 2 z
 
   -1,  0,  0, // 3 -x
    0, -1,  0, // 4 -y
    0,  0, -1, // 5 -z

	 0.70710f,0.70710f,0, // 6 left_up 45 sklon strechy 
	-0.70710f,0.70710f,0, // 7 right_up 45 sklon strechy 
	 0,0.70710f,0.70710f, // 8 back_up 45 sklon strechy 
	 0,0.70710f,-0.70710f,// 9 front_up 45 sklon strechy 
};

// steny z bodu
static int32_t p02_indices_front[] = { 1,0,2,3, -1 };
static int32_t p02_indices_front_up[] = { 2,3,4, -1 };
static int32_t p02_indices_left[] = { 1,8,2,5, -1 };
static int32_t p02_indices_left_up[] = { 2,5,4, -1 };
static int32_t p02_indices_right[] = { 0,7,3,6, -1 };
static int32_t p02_indices_right_up[] = { 3,6,4, -1 };
static int32_t p02_indices_back[] = { 7,8,6,5, -1 };
static int32_t p02_indices_back_up[] = { 6,5,4, -1 };
static int32_t p02_indices_bottom[] = { 0,1,7,8, -1 };

// normaly ke stenam 
static int normalIndex_p02_front[] = { 5, 5 };
static int normalIndex_p02_front_up[] = { 9 };
static int normalIndex_p02_left[] = { 0, 0 };
static int normalIndex_p02_left_up[] = { 6 };
static int normalIndex_p02_right[] = { 3, 3 };
static int normalIndex_p02_right_up[] = { 7 };
static int normalIndex_p02_back[] = { 2, 2 };
static int normalIndex_p02_back_up[] = { 8 };
static int normalIndex_p02_bottom[] = { 4, 4 };

// texturovaci souradnice pro jednotlive steny
static int32_t TexCoordIndex_p02_front[] = { 0,1,2,3, -1 };
static int32_t TexCoordIndex_p02_front_up[] = { 0,1,4, -1 };
static int32_t TexCoordIndex_p02_left[] = { 0,1,2,3, -1 };
static int32_t TexCoordIndex_p02_left_up[] = { 0,1,4, -1 };
static int32_t TexCoordIndex_p02_right[] = { 0,1,2,3, -1 };
static int32_t TexCoordIndex_p02_right_up[] = { 0,1,4, -1 };
static int32_t TexCoordIndex_p02_back[] = { 0,1,2,3, -1 };
static int32_t TexCoordIndex_p02_back_up[] = { 0,1,4, -1 };
static int32_t TexCoordIndex_p02_bottom[] = { 0,1,2,3, -1 };


// -----------------------------------------------------------------------------
// fce.MakeBuilding_p02 - vytvori budovu p02
// -----------------------------------------------------------------------------
SoSeparator *MakeBuilding_p02(float x, float y, float z, float alfa, TEXTURE texture) {

	// uzel oznacujici budovu
	SoSeparator *p02 = new SoSeparator();
	p02 -> ref();

	// umisteni budovy
  SoTransform *transform = new SoTransform;
  transform->translation.setValue(x,y,z);
  p02->addChild(transform);

	// velikost modelu budovy
  SoScale *scale = new SoScale;
  scale->scaleFactor.setValue(SCALE_X,SCALE_Y,SCALE_Z);
  p02->addChild(scale);

  // rotace modelu budovy
  SoRotationXYZ *myRotation = new SoRotationXYZ;     
  myRotation -> angle = alfa;
  myRotation -> axis = SoRotationXYZ::Y;
  p02 -> addChild(myRotation);

	// normaly modelu budovy
  SoNormal *myNormals = new SoNormal;
  myNormals -> vector.setValues(0, 10, p02_normal);
  p02 -> addChild(myNormals); 

	// prirazovani normal PER_FACE_INDEXED
  SoNormalBinding *myNormalBinding = new SoNormalBinding;
  myNormalBinding -> value = SoNormalBinding::PER_FACE_INDEXED;
  p02 -> addChild(myNormalBinding);
  
	// propojeni bodu modelu budovy
  SoCoordinate3 *coords = new SoCoordinate3;
  coords->point.setValues(0, sizeof(p02_vertices)/sizeof(float), p02_vertices);
  p02->addChild(coords);

	// 'vyrez textury' (tvar 2D pro krychly potrebujeme ctverec bereme celou textuturu)' 
  SoTextureCoordinate2 *texCoord = new SoTextureCoordinate2;
  texCoord->point.set1Value(0, SbVec2f(0,0));
  texCoord->point.set1Value(1, SbVec2f(1,0));
  texCoord->point.set1Value(2, SbVec2f(0,1));
  texCoord->point.set1Value(3, SbVec2f(1,1));
  texCoord->point.set1Value(4, SbVec2f(0.5,1));
  p02->addChild(texCoord);

  // front
  SoTexture2 *texture_p02 = new SoTexture2;
  p02 -> addChild(texture_p02);
	switch(texture) {
		case BRICK: texture_p02 -> filename.setValue("Texture/buildings/p02/brick/texture_p02_front.PNG"); break;
		case COLOR: texture_p02 -> filename.setValue("Texture/buildings/p02/color/texture_p02_front.PNG"); break;
		case STONE: texture_p02 -> filename.setValue("Texture/buildings/p02/stone/texture_p02_front.PNG"); break;
		case WOOD:  texture_p02 -> filename.setValue("Texture/buildings/p02/wood/texture_p02_front.PNG");  break;
	}

  SoIndexedTriangleStripSet *stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(p02_indices_front)/sizeof(int32_t), p02_indices_front);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_p02_front)/sizeof(int), normalIndex_p02_front);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_p02_front)/sizeof(int32_t), TexCoordIndex_p02_front); 
  p02->addChild(stripSet);


  // front_up
  texture_p02 = new SoTexture2;
  p02 -> addChild(texture_p02);
	switch(texture) {
		case BRICK: texture_p02 -> filename.setValue("Texture/buildings/p02/brick/texture_p02_front_up.PNG"); break;
		case COLOR: texture_p02 -> filename.setValue("Texture/buildings/p02/color/texture_p02_front_up.PNG"); break;
		case STONE: texture_p02 -> filename.setValue("Texture/buildings/p02/stone/texture_p02_front_up.PNG"); break;
		case WOOD:  texture_p02 -> filename.setValue("Texture/buildings/p02/wood/texture_p02_front_up.PNG");  break;
	}

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(p02_indices_front_up)/sizeof(int32_t), p02_indices_front_up);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_p02_front_up)/sizeof(int), normalIndex_p02_front_up);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_p02_front_up)/sizeof(int32_t), TexCoordIndex_p02_front_up); 
  p02->addChild(stripSet);


  // left
  texture_p02 = new SoTexture2;
  p02 -> addChild(texture_p02);
 	switch(texture) {
		case BRICK: texture_p02 -> filename.setValue("Texture/buildings/p02/brick/texture_p02_left.PNG"); break;
		case COLOR: texture_p02 -> filename.setValue("Texture/buildings/p02/color/texture_p02_left.PNG"); break;
		case STONE: texture_p02 -> filename.setValue("Texture/buildings/p02/stone/texture_p02_left.PNG"); break;
		case WOOD:  texture_p02 -> filename.setValue("Texture/buildings/p02/wood/texture_p02_left.PNG");  break;
	}

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(p02_indices_left)/sizeof(int32_t), p02_indices_left);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_p02_left)/sizeof(int), normalIndex_p02_left);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_p02_left)/sizeof(int32_t), TexCoordIndex_p02_left); 
  p02->addChild(stripSet);


  // left_up
  texture_p02 = new SoTexture2;
  p02 -> addChild(texture_p02);
	switch(texture) {
		case BRICK: texture_p02 -> filename.setValue("Texture/buildings/p02/brick/texture_p02_left_up.PNG"); break;
		case COLOR: texture_p02 -> filename.setValue("Texture/buildings/p02/color/texture_p02_left_up.PNG"); break;
		case STONE: texture_p02 -> filename.setValue("Texture/buildings/p02/stone/texture_p02_left_up.PNG"); break;
		case WOOD:  texture_p02 -> filename.setValue("Texture/buildings/p02/wood/texture_p02_left_up.PNG");  break;
	}

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(p02_indices_left_up)/sizeof(int32_t), p02_indices_left_up);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_p02_left_up)/sizeof(int), normalIndex_p02_left_up);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_p02_left_up)/sizeof(int32_t), TexCoordIndex_p02_left_up); 
  p02->addChild(stripSet);


  // right
  texture_p02 = new SoTexture2;
  p02 -> addChild(texture_p02);
	switch(texture) {
		case BRICK: texture_p02 -> filename.setValue("Texture/buildings/p02/brick/texture_p02_right.PNG"); break;
		case COLOR: texture_p02 -> filename.setValue("Texture/buildings/p02/color/texture_p02_right.PNG"); break;
		case STONE: texture_p02 -> filename.setValue("Texture/buildings/p02/stone/texture_p02_right.PNG"); break;
		case WOOD:  texture_p02 -> filename.setValue("Texture/buildings/p02/wood/texture_p02_right.PNG");  break;
	}

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(p02_indices_right)/sizeof(int32_t), p02_indices_right);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_p02_right)/sizeof(int), normalIndex_p02_right);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_p02_right)/sizeof(int32_t), TexCoordIndex_p02_right); 
  p02->addChild(stripSet);


  // right_up
  texture_p02 = new SoTexture2;
  p02 -> addChild(texture_p02);
	switch(texture) {
		case BRICK: texture_p02 -> filename.setValue("Texture/buildings/p02/brick/texture_p02_right_up.PNG"); break;
		case COLOR: texture_p02 -> filename.setValue("Texture/buildings/p02/color/texture_p02_right_up.PNG"); break;
		case STONE: texture_p02 -> filename.setValue("Texture/buildings/p02/stone/texture_p02_right_up.PNG"); break;
		case WOOD:  texture_p02 -> filename.setValue("Texture/buildings/p02/wood/texture_p02_right_up.PNG");  break;
	}

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(p02_indices_right_up)/sizeof(int32_t), p02_indices_right_up);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_p02_right_up)/sizeof(int), normalIndex_p02_right_up);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_p02_right_up)/sizeof(int32_t), TexCoordIndex_p02_right_up); 
  p02->addChild(stripSet);

  
  // back
  texture_p02 = new SoTexture2;
  p02 -> addChild(texture_p02);
	switch(texture) {
		case BRICK: texture_p02 -> filename.setValue("Texture/buildings/p02/brick/texture_p02_back.PNG"); break;
		case COLOR: texture_p02 -> filename.setValue("Texture/buildings/p02/color/texture_p02_back.PNG"); break;
		case STONE: texture_p02 -> filename.setValue("Texture/buildings/p02/stone/texture_p02_back.PNG"); break;
		case WOOD:  texture_p02 -> filename.setValue("Texture/buildings/p02/wood/texture_p02_back.PNG");  break;
	}

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(p02_indices_back)/sizeof(int32_t), p02_indices_back);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_p02_back)/sizeof(int), normalIndex_p02_back);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_p02_back)/sizeof(int32_t), TexCoordIndex_p02_back); 
  p02->addChild(stripSet);


  // back_up
  texture_p02 = new SoTexture2;
  p02 -> addChild(texture_p02);
	switch(texture) {
		case BRICK: texture_p02 -> filename.setValue("Texture/buildings/p02/brick/texture_p02_back_up.PNG"); break;
		case COLOR: texture_p02 -> filename.setValue("Texture/buildings/p02/color/texture_p02_back_up.PNG"); break;
		case STONE: texture_p02 -> filename.setValue("Texture/buildings/p02/stone/texture_p02_back_up.PNG"); break;
		case WOOD:  texture_p02 -> filename.setValue("Texture/buildings/p02/wood/texture_p02_back_up.PNG");  break;
	}

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(p02_indices_back_up)/sizeof(int32_t), p02_indices_back_up);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_p02_back_up)/sizeof(int), normalIndex_p02_back_up);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_p02_back_up)/sizeof(int32_t), TexCoordIndex_p02_back_up); 
  p02->addChild(stripSet);


  // bottom
  texture_p02 = new SoTexture2;
  p02 -> addChild(texture_p02);
  texture_p02 -> filename.setValue("texture_bottom.PNG");

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(p02_indices_bottom)/sizeof(int32_t), p02_indices_bottom);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_p02_bottom)/sizeof(int), normalIndex_p02_bottom);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_p02_bottom)/sizeof(int32_t), TexCoordIndex_p02_bottom); 
  p02->addChild(stripSet); 


	p02 -> unrefNoDelete();
	return p02;
}
// -----------------------------------------------------------------------------


// =============================================================================
// konec souboru building_p02.cpp
// =============================================================================