// =============================================================================
// soubor building_s01.cpp
// =============================================================================

#include"..\buildings.h"

// VERTICES 
// body v prostoru jejich propojenim vytvarim model budovy 
static float s01_vertices[][3] = {
	-0.5f, 0, -1,  // 0
	 0.5f, 0, -1,  // 1
   0.5f, 1, -1,  // 2
  -0.5f, 1, -1,  // 3

   0,  1.5, -1,  // 4
	 0,  1.5,  1,  // 5

	 0.5f, 1,  1,  // 6
  -0.5f, 1,  1,  // 7
  -0.5f, 0,  1,  // 8
   0.5f, 0,  1   // 9
};

// NORMALS
// normaly pro dobre osvetleni sten modelu budovy
static float s01_normal[][3] = {
  	1,  0,  0, // 0 x 
    0,  1,  0, // 1 y
    0,  0,  1, // 2 z
 
	 -1,  0,  0, // 3 -x
    0, -1,  0, // 4 -y
    0,  0, -1, // 5 -z

	 0.70710f,0.70710f,0, // 6 left_up 45 sklon strechy 
	-0.70710f,0.70710f,0, // 7 right_up 45 sklon strechy 
};

// steny z bodu
static int32_t s01_indices_front[] = { 1,0,2,3,4, -1 };
static int32_t s01_indices_left_up[] = { 4,5,2,6, -1 };
static int32_t s01_indices_left[] = { 2,6,1,9, -1 };
static int32_t s01_indices_right_up[] = { 4,5,3,7, -1 };
static int32_t s01_indices_right[] = { 3,7,0,8, -1 };
static int32_t s01_indices_back[] = { 5,6,7,9,8, -1 };
static int32_t s01_indices_bottom[] = { 0,1,8,9, -1 };

// normaly ke stenam
static int normalIndex_s01_front[] = { 5, 5, 5 };
static int normalIndex_s01_left_up[] = { 6, 6 };
static int normalIndex_s01_left[] = { 0, 0 };
static int normalIndex_s01_right_up[] = { 7, 7 };
static int normalIndex_s01_right[] = { 3, 3 };
static int normalIndex_s01_back[] = { 2, 2, 2 };
static int normalIndex_s01_bottom[] = { 4, 4 };

// texturovaci souradnice pro jednotlive steny
static int32_t TexCoordIndex_s7_front[] = { 1,0,6,4,5, -1 };
static int32_t TexCoordIndex_s7_left_up[] = { 0,1,2,3, -1 };
static int32_t TexCoordIndex_s7_left[] = { 2,3,0,1, -1 };
static int32_t TexCoordIndex_s7_right_up[] = { 0,1,2,3, -1 };
static int32_t TexCoordIndex_s7_right[] = { 2,3,0,1, -1 };
static int32_t TexCoordIndex_s7_back[] = { 5,4,6,0,1, -1 };
static int32_t TexCoordIndex_s7_bottom[] = { 0,1,2,3, -1 };


// -----------------------------------------------------------------------------
// fce.MakeBuilding_s01 - vytvori budovu s01
// -----------------------------------------------------------------------------
SoSeparator *MakeBuilding_s01(float x, float y, float z, float alfa, TEXTURE texture) {

	// uzel oznacujici budovu
	SoSeparator *s01 = new SoSeparator();
	s01 -> ref();

	// umisteni budovy
  SoTransform *transform = new SoTransform;
  transform->translation.setValue(x,y,z);
  s01->addChild(transform);

	// velikost modelu budovy
  SoScale *scale = new SoScale;
  scale->scaleFactor.setValue(SCALE_X,SCALE_Y,SCALE_Z);
  s01->addChild(scale);

  // rotace modelu budovy
  SoRotationXYZ *myRotation = new SoRotationXYZ;     
  myRotation -> angle = alfa;
  myRotation -> axis = SoRotationXYZ::Y;
  s01 -> addChild(myRotation);

	// normaly modelu budovy
  SoNormal *myNormals = new SoNormal;
  myNormals -> vector.setValues(0, 10, s01_normal);
  s01 -> addChild(myNormals); 

	// prirazovani normal PER_FACE_INDEXED
  SoNormalBinding *myNormalBinding = new SoNormalBinding;
  myNormalBinding -> value = SoNormalBinding::PER_FACE_INDEXED;
  s01 -> addChild(myNormalBinding);

	// propojeni bodu modelu budovy
  SoCoordinate3 *coords01 = new SoCoordinate3;
  coords01->point.setValues(0,sizeof(s01_vertices)/sizeof(float), s01_vertices);
  s01->addChild(coords01); 	

	// 'vyrez textury' (tvar 2D pro krychly potrebujeme ctverec bereme celou textuturu)'
  SoTextureCoordinate2 *texCoord = new SoTextureCoordinate2;
  texCoord->point.set1Value(0, SbVec2f(0,0));
  texCoord->point.set1Value(1, SbVec2f(1,0));
  texCoord->point.set1Value(2, SbVec2f(0,1));
  texCoord->point.set1Value(3, SbVec2f(1,1));

  texCoord->point.set1Value(4, SbVec2f(0,0.6666f)); // 2
  texCoord->point.set1Value(5, SbVec2f(0.5,1));
  texCoord->point.set1Value(6, SbVec2f(1,0.6666f)); // 3
  s01->addChild(texCoord);


  // front
  SoTexture2 *texture_s01 = new SoTexture2;
  s01 -> addChild(texture_s01);
	switch(texture) {
		case BRICK: texture_s01 -> filename.setValue("Texture/buildings/s01/brick/texture_s01_front.PNG"); break;
		case COLOR: texture_s01 -> filename.setValue("Texture/buildings/s01/color/texture_s01_front.PNG"); break;
		case STONE: texture_s01 -> filename.setValue("Texture/buildings/s01/stone/texture_s01_front.PNG"); break;
		case WOOD:  texture_s01 -> filename.setValue("Texture/buildings/s01/wood/texture_s01_front.PNG");  break;
	}

  SoIndexedTriangleStripSet *stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(s01_indices_front)/sizeof(int32_t), s01_indices_front);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_s01_front)/sizeof(int), normalIndex_s01_front);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_s7_front)/sizeof(int32_t), TexCoordIndex_s7_front); 
  s01->addChild(stripSet);

  // left_up
  texture_s01 = new SoTexture2;
  s01 -> addChild(texture_s01);
	switch(texture) {
		case BRICK: texture_s01 -> filename.setValue("Texture/buildings/s01/brick/texture_s01_left_up.PNG"); break;
		case COLOR: texture_s01 -> filename.setValue("Texture/buildings/s01/color/texture_s01_left_up.PNG"); break;
		case STONE: texture_s01 -> filename.setValue("Texture/buildings/s01/stone/texture_s01_left_up.PNG"); break;
		case WOOD:  texture_s01 -> filename.setValue("Texture/buildings/s01/wood/texture_s01_left_up.PNG");  break;
	}

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(s01_indices_left_up)/sizeof(int32_t), s01_indices_left_up);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_s01_left_up)/sizeof(int), normalIndex_s01_left_up);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_s7_left_up)/sizeof(int32_t), TexCoordIndex_s7_left_up); 
  s01->addChild(stripSet);


  // left
  texture_s01 = new SoTexture2;
  s01 -> addChild(texture_s01);
	switch(texture) {
		case BRICK: texture_s01 -> filename.setValue("Texture/buildings/s01/brick/texture_s01_left.PNG"); break;
		case COLOR: texture_s01 -> filename.setValue("Texture/buildings/s01/color/texture_s01_left.PNG"); break;
		case STONE: texture_s01 -> filename.setValue("Texture/buildings/s01/stone/texture_s01_left.PNG"); break;
		case WOOD:  texture_s01 -> filename.setValue("Texture/buildings/s01/wood/texture_s01_left.PNG");  break;
	}

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(s01_indices_left)/sizeof(int32_t), s01_indices_left);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_s01_left)/sizeof(int), normalIndex_s01_left);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_s7_left)/sizeof(int32_t), TexCoordIndex_s7_left); 
  s01->addChild(stripSet);


  // right_up
  texture_s01 = new SoTexture2;
  s01 -> addChild(texture_s01);
	switch(texture) {
		case BRICK: texture_s01 -> filename.setValue("Texture/buildings/s01/brick/texture_s01_right_up.PNG"); break;
		case COLOR: texture_s01 -> filename.setValue("Texture/buildings/s01/color/texture_s01_right_up.PNG"); break;
		case STONE: texture_s01 -> filename.setValue("Texture/buildings/s01/stone/texture_s01_right_up.PNG"); break;
		case WOOD:  texture_s01 -> filename.setValue("Texture/buildings/s01/wood/texture_s01_right_up.PNG");  break;
	}

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(s01_indices_right_up)/sizeof(int32_t), s01_indices_right_up);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_s01_right_up)/sizeof(int), normalIndex_s01_right_up);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_s7_right_up)/sizeof(int32_t), TexCoordIndex_s7_right_up); 
  s01->addChild(stripSet);


  // right
  texture_s01 = new SoTexture2;
  s01 -> addChild(texture_s01);
	switch(texture) {
		case BRICK: texture_s01 -> filename.setValue("Texture/buildings/s01/brick/texture_s01_right.PNG"); break;
		case COLOR: texture_s01 -> filename.setValue("Texture/buildings/s01/color/texture_s01_right.PNG"); break;
		case STONE: texture_s01 -> filename.setValue("Texture/buildings/s01/stone/texture_s01_right.PNG"); break;
		case WOOD:  texture_s01 -> filename.setValue("Texture/buildings/s01/wood/texture_s01_right.PNG");  break;
	}

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(s01_indices_right)/sizeof(int32_t), s01_indices_right);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_s01_right)/sizeof(int), normalIndex_s01_right);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_s7_right)/sizeof(int32_t), TexCoordIndex_s7_right); 
  s01->addChild(stripSet);

 
  // back
  texture_s01 = new SoTexture2;
  s01 -> addChild(texture_s01);
	switch(texture) {
		case BRICK: texture_s01 -> filename.setValue("Texture/buildings/s01/brick/texture_s01_back.PNG"); break;
		case COLOR: texture_s01 -> filename.setValue("Texture/buildings/s01/color/texture_s01_back.PNG"); break;
		case STONE: texture_s01 -> filename.setValue("Texture/buildings/s01/stone/texture_s01_back.PNG"); break;
		case WOOD:  texture_s01 -> filename.setValue("Texture/buildings/s01/wood/texture_s01_back.PNG");  break;
	}

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(s01_indices_back)/sizeof(int32_t), s01_indices_back);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_s01_back)/sizeof(int), normalIndex_s01_back);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_s7_back)/sizeof(int32_t), TexCoordIndex_s7_back); 
  s01->addChild(stripSet);


  // bottom
  texture_s01 = new SoTexture2;
  s01 -> addChild(texture_s01);
  texture_s01 -> filename.setValue("texture_bottom.PNG");

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(s01_indices_bottom)/sizeof(int32_t), s01_indices_bottom);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_s01_bottom)/sizeof(int), normalIndex_s01_bottom);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_s7_bottom)/sizeof(int32_t), TexCoordIndex_s7_bottom); 
  s01->addChild(stripSet);

	s01 -> unrefNoDelete();
	return s01;
}
// -----------------------------------------------------------------------------


// =============================================================================
// konec souboru building_s01.cpp
// =============================================================================