// =============================================================================
// soubor building_s05.cpp
// =============================================================================

#include"..\buildings.h"

// VERTICES 
// body v prostoru jejich propojenim vytvarim model budovy 
static float s05vertices[][3] = {
	-0.5f, 0, -1.5f,  // 0
	 0.5f, 0, -1.5f,  // 1
   0.5f, 2, -1.5f,  // 2
  -0.5f, 2, -1.5f,  // 3

	 0,  2.5, -1.5f,  // 4
	 0,  2.5,  1.5f,  // 5

	 0.5f, 2,  1.5f,  // 6
  -0.5f, 2,  1.5f,  // 7
  -0.5f, 0,  1.5f,  // 8
   0.5f, 0,  1.5f   // 9
};

// NORMALS
// normaly pro dobre osvetleni sten modelu budovy
static float s05_normal[][3] = {
  	1,  0,  0, // 0 x 
    0,  1,  0, // 1 y
    0,  0,  1, // 2 z
 
	 -1,  0,  0, // 3 -x
    0, -1,  0, // 4 -y
    0,  0, -1, // 5 -z

	 0.70710f,0.70710f,0, // 6 left_up 45 sklon strechy 
	-0.70710f,0.70710f,0, // 7 right_up 45 sklon strechy 
};

// steny z bodu
static int32_t s05indices_front[] = { 1,0,2,3,4, -1 };
static int32_t s05indices_left_up[] = { 4,5,2,6, -1 };
static int32_t s05indices_left[] = { 2,6,1,9, -1 };
static int32_t s05indices_right_up[] = { 4,5,3,7, -1 };
static int32_t s05indices_right[] = { 3,7,0,8, -1 };
static int32_t s05indices_back[] = { 5,6,7,9,8, -1 };
static int32_t s05indices_bottom[] = { 0,1,8,9, -1 };

// normaly ke stenam
static int normalIndex_s05front[] = { 5, 5, 5 };
static int normalIndex_s05left_up[] = { 6, 6 };
static int normalIndex_s05left[] = { 0, 0 };
static int normalIndex_s05right_up[] = { 7, 7 };
static int normalIndex_s05right[] = { 3, 3 };
static int normalIndex_s05back[] = { 2, 2, 2 };
static int normalIndex_s05bottom[] = { 4, 4 };

// texturovaci souradnice pro jednotlive steny
static int32_t TexCoordIndex_s05front[] = { 1,0,6,4,5, -1 };
static int32_t TexCoordIndex_s05left_up[] = { 0,1,2,3, -1 };
static int32_t TexCoordIndex_s05left[] = { 2,3,0,1, -1 };
static int32_t TexCoordIndex_s05right_up[] = { 0,1,2,3, -1 };
static int32_t TexCoordIndex_s05right[] = { 2,3,0,1, -1 };
static int32_t TexCoordIndex_s05back[] = { 5,4,6,0,1, -1 };
static int32_t TexCoordIndex_s05bottom[] = { 0,1,2,3, -1 };


// -----------------------------------------------------------------------------
// fce.MakeBuilding_s05 - vytvori budovu s05
// -----------------------------------------------------------------------------
SoSeparator *MakeBuilding_s05(float x, float y, float z, float alfa, TEXTURE texture) {

	// uzel oznacujici budovu
	SoSeparator *s05 = new SoSeparator();
	s05 -> ref();

	// umisteni budovy
  SoTransform *transform = new SoTransform;
  transform->translation.setValue(x,y,z);
  s05->addChild(transform);

	// velikost modelu budovy
  SoScale *scale = new SoScale;
  scale->scaleFactor.setValue(SCALE_X,SCALE_Y,SCALE_Z);
  s05->addChild(scale);

  // rotace modelu budovy
  SoRotationXYZ *myRotation = new SoRotationXYZ;     
  myRotation -> angle = alfa;
  myRotation -> axis = SoRotationXYZ::Y;
  s05 -> addChild(myRotation);

	// normaly modelu budovy
  SoNormal *myNormals = new SoNormal;
  myNormals -> vector.setValues(0, 10, s05_normal);
  s05 -> addChild(myNormals); 

	// prirazovani normal PER_FACE_INDEXED
  SoNormalBinding *myNormalBinding = new SoNormalBinding;
  myNormalBinding -> value = SoNormalBinding::PER_FACE_INDEXED;
  s05 -> addChild(myNormalBinding);

	// propojeni bodu modelu budovy
  SoCoordinate3 *coords1 = new SoCoordinate3;
  coords1->point.setValues(0,sizeof(s05vertices)/sizeof(float), s05vertices);
  s05->addChild(coords1); 	

	// 'vyrez textury' (tvar 2D pro krychly potrebujeme ctverec bereme celou textuturu)'
  SoTextureCoordinate2 *texCoord = new SoTextureCoordinate2;
  texCoord->point.set1Value(0, SbVec2f(0,0));
  texCoord->point.set1Value(1, SbVec2f(1,0));
  texCoord->point.set1Value(2, SbVec2f(0,1));
  texCoord->point.set1Value(3, SbVec2f(1,1));

  texCoord->point.set1Value(4, SbVec2f(0,0.6666f)); // 2
  texCoord->point.set1Value(5, SbVec2f(0.5,1));
  texCoord->point.set1Value(6, SbVec2f(1,0.6666f)); // 3
  s05->addChild(texCoord);


  // front
  SoTexture2 *texture_s05 = new SoTexture2;
  s05 -> addChild(texture_s05);
	switch(texture) {
		case BRICK: texture_s05 -> filename.setValue("Texture/buildings/s05/brick/texture_s05_front.PNG"); break;
		case COLOR: texture_s05 -> filename.setValue("Texture/buildings/s05/color/texture_s05_front.PNG"); break;
		case STONE: texture_s05 -> filename.setValue("Texture/buildings/s05/stone/texture_s05_front.PNG"); break;
		case WOOD:  texture_s05 -> filename.setValue("Texture/buildings/s05/wood/texture_s05_front.PNG");  break;
	}

  SoIndexedTriangleStripSet *stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(s05indices_front)/sizeof(int32_t), s05indices_front);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_s05front)/sizeof(int), normalIndex_s05front);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_s05front)/sizeof(int32_t), TexCoordIndex_s05front); 
  s05->addChild(stripSet);


  // left_up
  texture_s05 = new SoTexture2;
  s05 -> addChild(texture_s05);
	switch(texture) {
		case BRICK: texture_s05 -> filename.setValue("Texture/buildings/s05/brick/texture_s05_up.PNG"); break;
		case COLOR: texture_s05 -> filename.setValue("Texture/buildings/s05/color/texture_s05_up.PNG"); break;
		case STONE: texture_s05 -> filename.setValue("Texture/buildings/s05/stone/texture_s05_up.PNG"); break;
		case WOOD:  texture_s05 -> filename.setValue("Texture/buildings/s05/wood/texture_s05_up.PNG");  break;
	}

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(s05indices_left_up)/sizeof(int32_t), s05indices_left_up);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_s05left_up)/sizeof(int), normalIndex_s05left_up);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_s05left_up)/sizeof(int32_t), TexCoordIndex_s05left_up); 
  s05->addChild(stripSet);


  // left
  texture_s05 = new SoTexture2;
  s05 -> addChild(texture_s05);
	switch(texture) {
		case BRICK: texture_s05 -> filename.setValue("Texture/buildings/s05/brick/texture_s05_left.PNG"); break;
		case COLOR: texture_s05 -> filename.setValue("Texture/buildings/s05/color/texture_s05_left.PNG"); break;
		case STONE: texture_s05 -> filename.setValue("Texture/buildings/s05/stone/texture_s05_left.PNG"); break;
		case WOOD:  texture_s05 -> filename.setValue("Texture/buildings/s05/wood/texture_s05_left.PNG");  break;
	}

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(s05indices_left)/sizeof(int32_t), s05indices_left);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_s05left)/sizeof(int), normalIndex_s05left);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_s05left)/sizeof(int32_t), TexCoordIndex_s05left); 
  s05->addChild(stripSet);


  // right_up
  texture_s05 = new SoTexture2;
  s05 -> addChild(texture_s05);
	switch(texture) {
		case BRICK: texture_s05 -> filename.setValue("Texture/buildings/s05/brick/texture_s05_up.PNG"); break;
		case COLOR: texture_s05 -> filename.setValue("Texture/buildings/s05/color/texture_s05_up.PNG"); break;
		case STONE: texture_s05 -> filename.setValue("Texture/buildings/s05/stone/texture_s05_up.PNG"); break;
		case WOOD:  texture_s05 -> filename.setValue("Texture/buildings/s05/wood/texture_s05_up.PNG");  break;
	}

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(s05indices_right_up)/sizeof(int32_t), s05indices_right_up);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_s05right_up)/sizeof(int), normalIndex_s05right_up);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_s05right_up)/sizeof(int32_t), TexCoordIndex_s05right_up); 
  s05->addChild(stripSet);


  // right
  texture_s05 = new SoTexture2;
  s05 -> addChild(texture_s05);
	switch(texture) {
		case BRICK: texture_s05 -> filename.setValue("Texture/buildings/s05/brick/texture_s05_right.PNG"); break;
		case COLOR: texture_s05 -> filename.setValue("Texture/buildings/s05/color/texture_s05_right.PNG"); break;
		case STONE: texture_s05 -> filename.setValue("Texture/buildings/s05/stone/texture_s05_right.PNG"); break;
		case WOOD:  texture_s05 -> filename.setValue("Texture/buildings/s05/wood/texture_s05_right.PNG");  break;
	}

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(s05indices_right)/sizeof(int32_t), s05indices_right);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_s05right)/sizeof(int), normalIndex_s05right);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_s05right)/sizeof(int32_t), TexCoordIndex_s05right); 
  s05->addChild(stripSet);

 
  // back
  texture_s05 = new SoTexture2;
  s05 -> addChild(texture_s05);
	switch(texture) {
		case BRICK: texture_s05 -> filename.setValue("Texture/buildings/s05/brick/texture_s05_back.PNG"); break;
		case COLOR: texture_s05 -> filename.setValue("Texture/buildings/s05/color/texture_s05_back.PNG"); break;
		case STONE: texture_s05 -> filename.setValue("Texture/buildings/s05/stone/texture_s05_back.PNG"); break;
		case WOOD:  texture_s05 -> filename.setValue("Texture/buildings/s05/wood/texture_s05_back.PNG");  break;
	}

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(s05indices_back)/sizeof(int32_t), s05indices_back);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_s05back)/sizeof(int), normalIndex_s05back);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_s05back)/sizeof(int32_t), TexCoordIndex_s05back); 
  s05->addChild(stripSet);


  // bottom
  texture_s05 = new SoTexture2;
  s05 -> addChild(texture_s05);
  texture_s05 -> filename.setValue("texture_bottom.PNG");

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(s05indices_bottom)/sizeof(int32_t), s05indices_bottom);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_s05bottom)/sizeof(int), normalIndex_s05bottom);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_s05bottom)/sizeof(int32_t), TexCoordIndex_s05bottom); 
  s05->addChild(stripSet);

	s05 -> unrefNoDelete();
	return s05;
}
// -----------------------------------------------------------------------------


// =============================================================================
// konec souboru building_s05.cpp
// =============================================================================