// =============================================================================
// soubor building_s06.cpp
// =============================================================================

#include"..\buildings.h"

// VERTICES 
// body v prostoru jejich propojenim vytvarim model budovy 
static float s06vertices[][3] = {
	-0.5f, 0, -2,  // 0
	 0.5f, 0, -2,  // 1
   0.5f, 2, -2,  // 2
  -0.5f, 2, -2,  // 3

   0,  2.5, -2,  // 4
	 0,  2.5,  2,  // 5

	 0.5f, 2,  2,  // 6
  -0.5f, 2,  2,  // 7
  -0.5f, 0,  2,  // 8
   0.5f, 0,  2   // 9
};

// NORMALS
// normaly pro dobre osvetleni sten modelu budovy
static float s06_normal[][3] = {
  	1,  0,  0, // 0 x 
    0,  1,  0, // 1 y
    0,  0,  1, // 2 z
 
	 -1,  0,  0, // 3 -x
    0, -1,  0, // 4 -y
    0,  0, -1, // 5 -z

	 0.70710f,0.70710f,0, // 6 left_up 45 sklon strechy 
	-0.70710f,0.70710f,0, // 7 right_up 45 sklon strechy 
};

// steny z bodu
static int32_t s06indices_front[] = { 1,0,2,3,4, -1 };
static int32_t s06indices_left_up[] = { 4,5,2,6, -1 };
static int32_t s06indices_left[] = { 2,6,1,9, -1 };
static int32_t s06indices_right_up[] = { 4,5,3,7, -1 };
static int32_t s06indices_right[] = { 3,7,0,8, -1 };
static int32_t s06indices_back[] = { 5,6,7,9,8, -1 };
static int32_t s06indices_bottom[] = { 0,1,8,9, -1 };

// normaly ke stenam
static int normalIndex_s06front[] = { 5, 5, 5 };
static int normalIndex_s06left_up[] = { 6, 6 };
static int normalIndex_s06left[] = { 0, 0 };
static int normalIndex_s06right_up[] = { 7, 7 };
static int normalIndex_s06right[] = { 3, 3 };
static int normalIndex_s06back[] = { 2, 2, 2 };
static int normalIndex_s06bottom[] = { 4, 4 };

// texturovaci souradnice pro jednotlive steny
static int32_t TexCoordIndex_s06front[] = { 1,0,6,4,5, -1 };
static int32_t TexCoordIndex_s06left_up[] = { 0,1,2,3, -1 };
static int32_t TexCoordIndex_s06left[] = { 2,3,0,1, -1 };
static int32_t TexCoordIndex_s06right_up[] = { 0,1,2,3, -1 };
static int32_t TexCoordIndex_s06right[] = { 2,3,0,1, -1 };
static int32_t TexCoordIndex_s06back[] = { 5,4,6,0,1, -1 };
static int32_t TexCoordIndex_s06bottom[] = { 0,1,2,3, -1 };


// -----------------------------------------------------------------------------
// fce.MakeBuilding_s06 - vytvori budovu s06
// -----------------------------------------------------------------------------
SoSeparator *MakeBuilding_s06(float x, float y, float z, float alfa, TEXTURE texture) {

	// uzel oznacujici budovu
	SoSeparator *s06 = new SoSeparator();
	s06 -> ref();

	// umisteni budovy
  SoTransform *transform = new SoTransform;
  transform->translation.setValue(x,y,z);
  s06->addChild(transform);

	// velikost modelu budovy
  SoScale *scale = new SoScale;
  scale->scaleFactor.setValue(SCALE_X,SCALE_Y,SCALE_Z);
  s06->addChild(scale);

  // rotace modelu budovy
  SoRotationXYZ *myRotation = new SoRotationXYZ;     
  myRotation -> angle = alfa;
  myRotation -> axis = SoRotationXYZ::Y;
  s06 -> addChild(myRotation);

	// normaly modelu budovy
  SoNormal *myNormals = new SoNormal;
  myNormals -> vector.setValues(0, 10, s06_normal);
  s06 -> addChild(myNormals); 

	// prirazovani normal PER_FACE_INDEXED
  SoNormalBinding *myNormalBinding = new SoNormalBinding;
  myNormalBinding -> value = SoNormalBinding::PER_FACE_INDEXED;
  s06 -> addChild(myNormalBinding);

	// propojeni bodu modelu budovy
  SoCoordinate3 *coords1 = new SoCoordinate3;
  coords1->point.setValues(0,sizeof(s06vertices)/sizeof(float), s06vertices);
  s06->addChild(coords1); 	

	// 'vyrez textury' (tvar 2D pro krychly potrebujeme ctverec bereme celou textuturu)'
  SoTextureCoordinate2 *texCoord = new SoTextureCoordinate2;
  texCoord->point.set1Value(0, SbVec2f(0,0));
  texCoord->point.set1Value(1, SbVec2f(1,0));
  texCoord->point.set1Value(2, SbVec2f(0,1));
  texCoord->point.set1Value(3, SbVec2f(1,1));

  texCoord->point.set1Value(4, SbVec2f(0,0.6666f)); // 2
  texCoord->point.set1Value(5, SbVec2f(0.5,1));
  texCoord->point.set1Value(6, SbVec2f(1,0.6666f)); // 3
  s06->addChild(texCoord);


  // front
  SoTexture2 *texture_s06 = new SoTexture2;
  s06 -> addChild(texture_s06);
	switch(texture) {
		case BRICK: texture_s06 -> filename.setValue("Texture/buildings/s06/brick/texture_s06_front.PNG"); break;
		case COLOR: texture_s06 -> filename.setValue("Texture/buildings/s06/color/texture_s06_front.PNG"); break;
		case STONE: texture_s06 -> filename.setValue("Texture/buildings/s06/stone/texture_s06_front.PNG"); break;
		case WOOD:  texture_s06 -> filename.setValue("Texture/buildings/s06/wood/texture_s06_front.PNG");  break;
	}

  SoIndexedTriangleStripSet *stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(s06indices_front)/sizeof(int32_t), s06indices_front);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_s06front)/sizeof(int), normalIndex_s06front);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_s06front)/sizeof(int32_t), TexCoordIndex_s06front); 
  s06->addChild(stripSet);


  // left_up
  texture_s06 = new SoTexture2;
  s06 -> addChild(texture_s06);
	switch(texture) {
		case BRICK: texture_s06 -> filename.setValue("Texture/buildings/s06/brick/texture_s06_up.PNG"); break;
		case COLOR: texture_s06 -> filename.setValue("Texture/buildings/s06/color/texture_s06_up.PNG"); break;
		case STONE: texture_s06 -> filename.setValue("Texture/buildings/s06/stone/texture_s06_up.PNG"); break;
		case WOOD:  texture_s06 -> filename.setValue("Texture/buildings/s06/wood/texture_s06_up.PNG");  break;
	}

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(s06indices_left_up)/sizeof(int32_t), s06indices_left_up);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_s06left_up)/sizeof(int), normalIndex_s06left_up);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_s06left_up)/sizeof(int32_t), TexCoordIndex_s06left_up); 
  s06->addChild(stripSet);


  // left
  texture_s06 = new SoTexture2;
  s06 -> addChild(texture_s06);
	switch(texture) {
		case BRICK: texture_s06 -> filename.setValue("Texture/buildings/s06/brick/texture_s06_left.PNG"); break;
		case COLOR: texture_s06 -> filename.setValue("Texture/buildings/s06/color/texture_s06_left.PNG"); break;
		case STONE: texture_s06 -> filename.setValue("Texture/buildings/s06/stone/texture_s06_left.PNG"); break;
		case WOOD:  texture_s06 -> filename.setValue("Texture/buildings/s06/wood/texture_s06_left.PNG");  break;
	}

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(s06indices_left)/sizeof(int32_t), s06indices_left);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_s06left)/sizeof(int), normalIndex_s06left);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_s06left)/sizeof(int32_t), TexCoordIndex_s06left); 
  s06->addChild(stripSet);


  // right_up
  texture_s06 = new SoTexture2;
  s06 -> addChild(texture_s06);
	switch(texture) {
		case BRICK: texture_s06 -> filename.setValue("Texture/buildings/s06/brick/texture_s06_up.PNG"); break;
		case COLOR: texture_s06 -> filename.setValue("Texture/buildings/s06/color/texture_s06_up.PNG"); break;
		case STONE: texture_s06 -> filename.setValue("Texture/buildings/s06/stone/texture_s06_up.PNG"); break;
		case WOOD:  texture_s06 -> filename.setValue("Texture/buildings/s06/wood/texture_s06_up.PNG");  break;
	}

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(s06indices_right_up)/sizeof(int32_t), s06indices_right_up);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_s06right_up)/sizeof(int), normalIndex_s06right_up);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_s06right_up)/sizeof(int32_t), TexCoordIndex_s06right_up); 
  s06->addChild(stripSet);


  // right
  texture_s06 = new SoTexture2;
  s06 -> addChild(texture_s06);
	switch(texture) {
		case BRICK: texture_s06 -> filename.setValue("Texture/buildings/s06/brick/texture_s06_right.PNG"); break;
		case COLOR: texture_s06 -> filename.setValue("Texture/buildings/s06/color/texture_s06_right.PNG"); break;
		case STONE: texture_s06 -> filename.setValue("Texture/buildings/s06/stone/texture_s06_right.PNG"); break;
		case WOOD:  texture_s06 -> filename.setValue("Texture/buildings/s06/wood/texture_s06_right.PNG");  break;
	}

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(s06indices_right)/sizeof(int32_t), s06indices_right);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_s06right)/sizeof(int), normalIndex_s06right);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_s06right)/sizeof(int32_t), TexCoordIndex_s06right); 
  s06->addChild(stripSet);

 
  // back
  texture_s06 = new SoTexture2;
  s06 -> addChild(texture_s06);
	switch(texture) {
		case BRICK: texture_s06 -> filename.setValue("Texture/buildings/s06/brick/texture_s06_back.PNG"); break;
		case COLOR: texture_s06 -> filename.setValue("Texture/buildings/s06/color/texture_s06_back.PNG"); break;
		case STONE: texture_s06 -> filename.setValue("Texture/buildings/s06/stone/texture_s06_back.PNG"); break;
		case WOOD:  texture_s06 -> filename.setValue("Texture/buildings/s06/wood/texture_s06_back.PNG");  break;
	}

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(s06indices_back)/sizeof(int32_t), s06indices_back);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_s06back)/sizeof(int), normalIndex_s06back);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_s06back)/sizeof(int32_t), TexCoordIndex_s06back); 
  s06->addChild(stripSet);


  // bottom
  texture_s06 = new SoTexture2;
  s06 -> addChild(texture_s06);
  texture_s06 -> filename.setValue("texture_bottom.PNG");

  stripSet = new SoIndexedTriangleStripSet;
  stripSet->coordIndex.setValues(0, sizeof(s06indices_bottom)/sizeof(int32_t), s06indices_bottom);
  stripSet->normalIndex.setValues(0, sizeof(normalIndex_s06bottom)/sizeof(int), normalIndex_s06bottom);
  stripSet->textureCoordIndex.setValues(0, sizeof(TexCoordIndex_s06bottom)/sizeof(int32_t), TexCoordIndex_s06bottom); 
  s06->addChild(stripSet);

	s06 -> unrefNoDelete();
	return s06;
}
// -----------------------------------------------------------------------------


// =============================================================================
// konec souboru building_s06.cpp
// =============================================================================