// =============================================================================
// soubor structures.h
// =============================================================================

#ifndef structures_h
#define structures_h

#include <stdlib.h>
#include <time.h>

#include "..\Buildings\MakeBuilding.h"

// typ pozice struktury
typedef struct position {
  float x_min;
  float z_min;
  float x_max;
  float z_max;
} POSITION;

// typ selektu podle ceho se ma vybyrat
typedef enum {BY_XYZ,
              BY_DOWN_MIN,BY_DOWN_MAX,
							BY_HIGH_MIN,BY_HIGH_MAX,
							BY_DMIN_HMIN,BY_DMIN_HMAX,
							BY_DMAX_HMIN,BY_DMAX_HMAX,
} BY;

// velikost povrchu pod mest.strukt.
#define MAX_GROUND 16.f

// fce.vyznaci silnici pro danou strukturu
void MarkRoad(int IPs);

// typ podle ceho se ma vybyrat textura nanesena na 1/16 povrchu mest.strukt.
typedef enum {
	GRASS,

	PAWEMENT, 
	PAWEMENT_45L_L,PAWEMENT_45L_R,PAWEMENT_45R_L,PAWEMENT_45R_R,
	PAWEMENT_45C_U,PAWEMENT_45C_D,PAWEMENT_45C_L,PAWEMENT_45C_R,

  SIDEWALK_UP,SIDEWALK_DOWN,SIDEWALK_LEFT,SIDEWALK_RIGHT,
	SIDEWALK_C_LU,SIDEWALK_C_LD,SIDEWALK_C_RU,SIDEWALK_C_RD,

	ROAD01_VERT,ROAD01_HORIZ,ROAD01_CROSS,
	ROAD01_C02_L,ROAD01_C02_R,ROAD01_C02_D,ROAD01_C02_U,
	ROAD01_CROSS_TL,ROAD01_CROSS_TR,ROAD01_CROSS_TD,ROAD01_CROSS_TU,
	ROAD01_90DL,ROAD01_90DR,ROAD01_90UL,ROAD01_90UR,
	ROAD01_45L_L,ROAD01_45L_M,ROAD01_45L_R,
	ROAD01_45R_L,ROAD01_45R_M,ROAD01_45R_R,
  ROAD01_45C02_LU,ROAD01_45C02_LD,ROAD01_45C02_RU,ROAD01_45C02_RD,
	ROAD01_45_C02_LU,ROAD01_45_C02_LD,ROAD01_45_C02_RU,ROAD01_45_C02_RD,
	ROAD01_45_CP_LU_LD,ROAD01_45_CP_LD_LU,ROAD01_45_CP_RU_LU,ROAD01_45_CP_RD_LD,
	ROAD01_45_CP_LU_RU,ROAD01_45_CP_LD_RD,ROAD01_45_CP_RU_RD,ROAD01_45_CP_RD_RU,
  ROAD01_45C02L_DU,ROAD01_45C02L_UD,ROAD01_45C02R_DU,ROAD01_45C02R_UD,
	ROAD01_45C02D_LR,ROAD01_45C02D_RL,ROAD01_45C02U_LR,ROAD01_45C02U_RL,

	ROAD01_45TL_11,ROAD01_45TL_12,
	ROAD01_45TL_21,ROAD01_45TL_22,
	ROAD01_45TL_31,ROAD01_45TL_32,
	ROAD01_45TL_41,ROAD01_45TL_42,
	ROAD01_45TR_11,ROAD01_45TR_12,
	ROAD01_45TR_21,ROAD01_45TR_22,
	ROAD01_45TR_31,ROAD01_45TR_32,
	ROAD01_45TR_41,ROAD01_45TR_42,

	ROAD02_45L_L,ROAD02_45L_M,ROAD02_45L_R,
	ROAD02_45R_L,ROAD02_45R_M,ROAD02_45R_R,
	ROAD02_VERT,ROAD02_HORIZ,ROAD02_CROSS,
	ROAD02_90T_DL,ROAD02_90T_DR,ROAD02_90T_UL,ROAD02_90T_UR,
	ROAD02_45C_U,ROAD02_45C_D,ROAD02_45C_L,ROAD02_45C_R,
	ROAD02_45C_LU,ROAD02_45C_LD,ROAD02_45C_RU,ROAD02_45C_RD,
	ROAD02_CROSS_45L11,ROAD02_CROSS_45L12,
	ROAD02_CROSS_45L21,ROAD02_CROSS_45L22,
	ROAD02_CROSS_45L31,ROAD02_CROSS_45L32,
	ROAD02_CROSS_45L41,ROAD02_CROSS_45L42,
	ROAD02_CROSS_45R11,ROAD02_CROSS_45R12,
	ROAD02_CROSS_45R21,ROAD02_CROSS_45R22,
	ROAD02_CROSS_45R31,ROAD02_CROSS_45R32,
	ROAD02_CROSS_45R41,ROAD02_CROSS_45R42,
} GROUND;

// fce.vraci index co mam byt na povrchu
// GRASS - trava
// PAWEMENT,SIDEWALK - chodnik
// ROAD02 - silnice - obousmerna
// ROAD01 - silnice s chodniky - jednosmerna
GROUND OnGround(int x, int y);

// fce.vyznaci silnici pro RandomTownStructure
GROUND OnRandomGround(int x,int y);

// prevod stupnu na radiany
float degree_to_radian(float x);

// fce. vytvori povrch struktury
SoSeparator* CreateGround(int c);


// fce.vytvari pozadovanou mestskou strukturu 

// centra
SoSeparator* MakeCentre00();
SoSeparator* MakeCentre01();
SoSeparator* MakeCentre02();
SoSeparator* MakeCentre03();

// ctvrte
SoSeparator* MakeDistrict00();
SoSeparator* MakeDistrict01();
SoSeparator* MakeDistrict02();
SoSeparator* MakeDistrict03();

// namesti
SoSeparator* MakeSquare00();
SoSeparator* MakeSquare01();
SoSeparator* MakeSquare02();
SoSeparator* MakeSquare03();

// predmesti
SoSeparator* MakeSuburb00();
SoSeparator* MakeSuburb01();
SoSeparator* MakeSuburb02();
SoSeparator* MakeSuburb03();

// nahodne generovana
SoSeparator* MakeRandomStructure();

// fce.pro vyber budov podle presne danych rozmeru
int SelectBuilding(BY by,int pre_building, float x, float y, float z);

// fce.pro vyber budov podle presne danych rozmeru
int by_xyz(int pre_building, float x, float y, float z);

// fce.pro vyber budov od danych rozmeru MIN podstavy
int by_min_down(int pre_building, float x, float z);

// fce.pro vyber budov do danych rozmeru MAX podstavy
int by_max_down(int pre_building, float x, float z);

// fce.pro vyber budov podle od dane vysky MIN vyska
int by_min_high(int pre_building, float y);

// fce.pro vyber budov podle do dane vysky MAX vyska
int by_max_high(int pre_building, float y);

// fce.pro vyber budov podle danych rozmeru MAX podstavy a MIN vysky
int by_max_down_and_min_high(int pre_building, float x, float y, float z);

// fce.pro vyber budov podle danych rozmeru MAX podstavy a MAX vysky
int by_max_down_and_max_high(int pre_building, float x, float y, float z);

// fce.pro vyber budov podle danych rozmeru MIN podstavy a MIN vysky
int by_min_down_and_min_high(int pre_building, float x, float y, float z);

// fce.pro vyber budov podle danych rozmeru MIN podstavy a MAX vysky
int by_min_down_and_max_high(int pre_building, float x, float y, float z);

// fce.pro vyber budov podle danych rozmeru  od do xyz
int SB_from_to_xyz(int pre_building, float x1, float x2, float y1, float y2, float z1, float z2);


// fce.prehodi texturu na nasledujici texturu
TEXTURE NextTexture(TEXTURE texture);

// fce.pseudonahodne zvoli texturu 
TEXTURE RandTexture();

// fce.vybere pseudonahodne (rand()) texturu
int RandBuilding();

// fce. umisti danou budovu do struktury
SoSeparator* PutBuilding(int building, float x, float z, float alfa, TEXTURE texture);

#endif

// =============================================================================
// konec souboru structures.h
// =============================================================================