// =============================================================================
// soubor TerrainModif.cpp
// =============================================================================

#include "TerrainModif.h"

// -----------------------------------------------------------------------------
// prevodni fce. stupne na radiany
// -----------------------------------------------------------------------------
float PI03degree_to_radian (float x) {
  const double PI = 3.1415;
  return (float) (PI/180)*x; 
}
// -----------------------------------------------------------------------------


// -----------------------------------------------------------------------------
// fce. prepocte souradnice bodu(rohu) podle rotacaniho uhlu za pomoci stredu
// -----------------------------------------------------------------------------
SbVec2f PI03RotatePodstava(int b, int x,int y,float rot_alfa,int x1,int y1,int x2,int y2) {

  float new_x = 0.0;
  float new_y = 0.0;

  float neorot_stred_x;
  float neorot_stred_y;
  float orot_stred_x;
  float orot_stred_y;
	
  // rotace
  new_x = x * (float) cos(PI03degree_to_radian(rot_alfa)) 
	      - y * (float) sin(PI03degree_to_radian(rot_alfa));
  new_y = x * (float) sin(PI03degree_to_radian(rot_alfa)) 
	      + y * (float) cos(PI03degree_to_radian(rot_alfa));

  // translace
  // stredy podle nich se provadi translace 
  // stred musi zustat na stejnem miste i po orortovani
  neorot_stred_x = (float) (x2-x1)/2 + x1;
  neorot_stred_y = (float) (y2-y1)/2 + y1;

  orot_stred_x = neorot_stred_x * (float) cos(PI03degree_to_radian(rot_alfa)) 
	           - neorot_stred_y * (float) sin(PI03degree_to_radian(rot_alfa));
  orot_stred_y = neorot_stred_x * (float) sin(PI03degree_to_radian(rot_alfa)) 
	           + neorot_stred_y * (float) cos(PI03degree_to_radian(rot_alfa));

  new_x = new_x + (neorot_stred_x - orot_stred_x);
  new_y = new_y + (neorot_stred_y - orot_stred_y);

  return (SbVec2f(new_x,new_y));
}
// -----------------------------------------------------------------------------


// -----------------------------------------------------------------------------
// fce.dopocte druhou souradnici bodu na usece A-B podle paramet.vyjadreni primky
// -----------------------------------------------------------------------------
int PI03get_second_coordinate(int y,SbVec2f A,SbVec2f B) {
float a1,a2,b1,b2,u1,u2,t,x;
a1 = A[0]; a2 = A[1];
b1 = B[0]; b2 = B[1];
u1 = a1 - b1;
u2 = a2 - b2;
if(u2 == 0) t = 0;
else t = (y - a2)/u2;
x = a1 + t*u1;
return (int)x;
}
// -----------------------------------------------------------------------------


// -----------------------------------------------------------------------------
// fce.modifikuje teren pod budovou na prumernou vysku a vraci prumernou vysku
// -----------------------------------------------------------------------------
float PI03modif(PI03TerrainPatch *tp, int x1, int y1, int x2, int y2, float alfa) {

  int y_min,y_max,x_min,x_max,x_min_y_kolize,x_max_y_kolize;
  const int offset = 1;

  // offset od budovi
  x1 = x1 - offset;
  y1 = y1 - offset;
  x2 = x2 + offset;
  y2 = y2 + offset;

  // rotace (prepocet souradnic bodu podle uhlu orotovani)
  SbVec2f &A = {PI03RotatePodstava(1,x1,y1,alfa,x1,y1,x2,y2)};  
  SbVec2f &B = {PI03RotatePodstava(1,x1,y2,alfa,x1,y1,x2,y2)};
  SbVec2f &C = {PI03RotatePodstava(1,x2,y1,alfa,x1,y1,x2,y2)};
  SbVec2f &D = {PI03RotatePodstava(1,x2,y2,alfa,x1,y1,x2,y2)};

  // pruchod pomoci primkoveho algoritmu
  y_min = (int)A[1]; 
  y_max = (int)D[1];
  x_min_y_kolize = (int)B[1];  
  x_max_y_kolize = (int)C[1];

  // promene pro vypocet prumerne vyzky
  float sum_heigth=0.f;
  int count_heigth=0;

  // prochazim dane uzemi a pocitam si vysky
  for(int yh = y_min; yh <= y_max; yh++) {
    if(yh<x_min_y_kolize) x_min = PI03get_second_coordinate(yh, A, B);
    else x_min = PI03get_second_coordinate(yh, B, D);
    if(yh<x_max_y_kolize) x_max = PI03get_second_coordinate(yh, A, C);
    else x_max = PI03get_second_coordinate(yh, C, D);
    for(int xh = x_min; xh <= x_max; xh++) { 
   	  sum_heigth += tp -> HMap[tp ->getHmapIndex(xh,yh)];  
      count_heigth++;
	}
  }

  // prumerna vyska
  float avarege_heigth = sum_heigth/count_heigth;

  // konecne modifikuji teren na prumernou vysku 
  for(int y = y_min; y <= y_max; y++) {
    if(y<x_min_y_kolize) x_min = PI03get_second_coordinate(y, A, B);
    else x_min = PI03get_second_coordinate(y, B, D);
    if(y<x_max_y_kolize) x_max = PI03get_second_coordinate(y, A, C);
    else x_max = PI03get_second_coordinate(y, C, D);
    for(int x = x_min; x <= x_max; x++) 
   	  tp -> HMap[tp ->getHmapIndex(x,y)] = avarege_heigth;  
  }
  
  return(avarege_heigth);
}
// -----------------------------------------------------------------------------


// =============================================================================
// konec souboru TerrainModif.cpp
// =============================================================================